@@ -150,7 +150,7 @@ struct TORCH_API FunctionalTensorWrapper : public c10::TensorImpl {
   void set__impl(const FunctionalTensorWrapper* other);
 
   // Custom implementation of resize_storage_bytes_(self, new_size)
-  void storage_resize_(c10::SymInt new_size);
+  void storage_resize_(const c10::SymInt& new_size);
 
   // Returns whether the current tensor's data was ever mutated
   bool has_data_mutation();
@@ -161,6 +161,16 @@ struct TORCH_API FunctionalTensorWrapper : public c10::TensorImpl {
     return was_storage_changed_;
   }
 
+  void set_storage_changed() {
+    was_storage_changed_ = true;
+  }
+
+  // A FunctionalTensor is considered a base if its not a view of another
+  // tensor.
+  bool isBaseTensor() const {
+    return view_metas_.empty();
+  }
+
   c10::SymInt get_storage_size(bool before) {
     return functional_storage_impl()->get_storage_size(before);
   }
@@ -222,6 +232,7 @@ struct TORCH_API FunctionalTensorWrapper : public c10::TensorImpl {
   c10::SymIntArrayRef sym_strides_custom() const override;
   c10::SymInt sym_storage_offset_custom() const override;
   c10::Device device_custom() const override;
+  c10::Layout layout_impl() const override;
 
  private:
   const char* tensorimpl_type_name() const override;
@@ -285,6 +296,8 @@ TORCH_API inline FunctionalTensorWrapper* unsafeGetFunctionalWrapper(
   return functional_impl;
 }
 
+TORCH_API bool isBaseTensor(const at::Tensor& tensor);
+
 TORCH_API bool isFunctionalTensor(const at::Tensor& tensor);
 TORCH_API bool isFunctionalTensor(const std::optional<Tensor>& t);
 TORCH_API bool isFunctionalTensor(
@@ -342,6 +355,13 @@ TORCH_API void propagate_xla_data(
     const ITensorListRef functional_tensor,
     ITensorListRef other);
 
+TORCH_API void propagate_xla_data_direct(
+    const Tensor& tensor,
+    const Tensor& other);
+TORCH_API void propagate_xla_data_direct(
+    const ITensorListRef tensor,
+    ITensorListRef other);
+
 Tensor create_functional_tensor_with_view_meta(
     const Tensor& view_to_wrap,
     const Tensor& base,
