@@ -262,9 +262,8 @@ struct CheckAlmostAllZeroStrides {
   static inline bool eval(const int64_t* strides) {
     // N is dim index: N -> dim0, N-1 -> dim1, ...
     // non_zero_stride_dim should be out_dims - dim
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    bool output;
-    if (N == non_zero_stride_dim) {
+    bool output = false;
+    if constexpr (N == non_zero_stride_dim) {
       output = is_contiguous_stride<scalar_t, index_t, interp_size>(strides);
     } else {
       output = is_zero_stride<2 * interp_size>(strides);
@@ -480,8 +479,8 @@ void cpu_upsample_nearest_channels_last(
   int64_t channels =  input_sizes[1];
   int64_t input_depth = (ndim == 5) ? input_sizes[2] : 1;
   int64_t output_depth = (ndim == 5) ? output_sizes[2] : 1;
-  int64_t input_height = (ndim >= 4) ? input_sizes[ndim - 2] : 1;
-  int64_t output_height = (ndim >= 4) ? output_sizes[ndim - 2] : 1;
+  int64_t input_height = input_sizes[ndim - 2];
+  int64_t output_height = output_sizes[ndim - 2];
   int64_t input_width = input_sizes[ndim - 1];
   int64_t output_width = output_sizes[ndim - 1];
   int64_t numel = output.numel();
@@ -586,8 +585,8 @@ void cpu_upsample_linear_channels_last(
   int64_t channels =  input_sizes[1];
   int64_t input_depth = (ndim == 5) ? input_sizes[2] : 1;
   int64_t output_depth = (ndim == 5) ? output_sizes[2] : 1;
-  int64_t input_height = (ndim >= 4) ? input_sizes[ndim - 2] : 1;
-  int64_t output_height = (ndim >= 4) ? output_sizes[ndim - 2] : 1;
+  int64_t input_height = input_sizes[ndim - 2];
+  int64_t output_height = output_sizes[ndim - 2];
   int64_t input_width = input_sizes[ndim - 1];
   int64_t output_width = output_sizes[ndim - 1];
 
@@ -607,8 +606,7 @@ void cpu_upsample_linear_channels_last(
           h * input_width * channels + w * channels;
     };
 
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    int64_t ih0, ih1, iw0, iw1;
+    int64_t ih0 = 0, ih1 = 0, iw0 = 0, iw1 = 0;
     opmath_t h0lambda, h1lambda, w0lambda, w1lambda;
     for (const auto n : c10::irange(begin, end)) {
       for (const auto oh : c10::irange(output_height)) {
@@ -657,8 +655,7 @@ void cpu_upsample_linear_channels_last(
           h * input_width * channels + w * channels;
     };
 
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    int64_t id0, id1, ih0, ih1, iw0, iw1;
+    int64_t id0 = 0, id1 = 0, ih0 = 0, ih1 = 0, iw0 = 0, iw1 = 0;
     opmath_t d0lambda, d1lambda, h0lambda, h1lambda, w0lambda, w1lambda;
     for (const auto n : c10::irange(begin, end)) {
       for (const auto od : c10::irange(output_depth)) {
@@ -806,7 +803,7 @@ struct HelperInterpBase {
         scale, i, align_corners, /*cubic=*/cubic);
 
     scalar_t lambda;
-    int64_t input_index;
+    int64_t input_index = 0;
     guard_index_and_lambda(real_input_index, input_size, input_index, lambda);
 
     const auto support = static_cast<int64_t>(max_interp_size * 0.5);
@@ -866,7 +863,7 @@ struct HelperInterpBase {
     std::vector<Tensor> output;
 
     scalar_t support;
-    int max_interp_size;
+    int max_interp_size = 0;
     if (antialias) {
         support = (scale >= 1.0) ? (interp_size * 0.5) * scale : interp_size * 0.5;
         max_interp_size = (int) std::ceil(support) * 2 + 1;
@@ -908,7 +905,7 @@ struct HelperInterpBase {
 
     scalar_t wt_max = 0.0;
     for (const auto i : c10::irange(output_size)) {
-      int64_t xmin, xsize;
+      int64_t xmin = 0, xsize = 0;
       scalar_t wt_max_i;
       if (antialias) {
         wt_max_i = HelperInterpBase::_compute_indices_min_size_weights_aa(
@@ -987,21 +984,20 @@ struct HelperInterpBase {
   template <typename aa_filter_fn_t>
   static inline std::tuple<std::vector<Tensor>, int, unsigned int> _compute_index_ranges_int16_weights(
     int64_t input_size, int64_t output_size, int64_t stride, int64_t ndims,
-    int64_t reshape_dim, bool align_corners, const std::optional<double> opt_scale,
+    int64_t reshape_dim, bool align_corners, const std::optional<double>& opt_scale,
     int interp_size, aa_filter_fn_t aa_filter_fn, bool antialias, bool align_i32=false
   ) {
 
     double scale = area_pixel_compute_scale<double>(
         input_size, output_size, align_corners, opt_scale);
 
-    std::vector<Tensor> indices_weights;
-    double wt_max;
-    std::tie(indices_weights, interp_size, wt_max) = HelperInterpBase::_compute_index_ranges_weights<double, aa_filter_fn_t, sizeof(int16_t)>(
+    auto [indices_weights, aligned_interp_size, wt_max] = HelperInterpBase::_compute_index_ranges_weights<double, aa_filter_fn_t, sizeof(int16_t)>(
         input_size, output_size, stride, ndims, reshape_dim, scale, interp_size, aa_filter_fn, antialias, align_corners);
+    interp_size = aligned_interp_size;
 
     // Rescale float weights to int16 and compute weights precision
     auto weights_f64 = indices_weights[3];
-    double * data_f64 = weights_f64.data_ptr<double>();
+    double * data_f64 = weights_f64. template data_ptr<double>();
 
     unsigned int weights_precision = 0;
     for (weights_precision = 0; weights_precision < 22; ++weights_precision) {
@@ -1012,7 +1008,6 @@ struct HelperInterpBase {
 
     // Rescale float values to int16
     int16_t * data_i16 = (int16_t *) data_f64;
-    auto aligned_interp_size = interp_size;
 
     if (align_i32) {
       // We should respect int32 alignment as we will load int16 data as int32
@@ -1072,11 +1067,10 @@ struct HelperInterpNearest : public HelperInterpBase {
   static inline std::vector<Tensor> compute_indices_weights(
     at::ScalarType scalar_type,
     int64_t input_size, int64_t output_size, int64_t stride, int64_t ndims,
-    int64_t reshape_dim, bool align_corners, const std::optional<double> opt_scale
+    int64_t reshape_dim, bool align_corners, const std::optional<double>& opt_scale
   ) {
 
     TORCH_INTERNAL_ASSERT(!align_corners);
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
     std::vector<Tensor> output;
     HelperInterpNearest::init_indices_weights(
       scalar_type, output, output_size, ndims, reshape_dim, HelperInterpNearest::interp_size);
@@ -1123,11 +1117,10 @@ struct HelperInterpNearestExact : public HelperInterpNearest {
   static inline std::vector<Tensor> compute_indices_weights(
     at::ScalarType scalar_type,
     int64_t input_size, int64_t output_size, int64_t stride, int64_t ndims,
-    int64_t reshape_dim, bool align_corners, const std::optional<double> opt_scale
+    int64_t reshape_dim, bool align_corners, const std::optional<double>& opt_scale
   ) {
 
     TORCH_INTERNAL_ASSERT(!align_corners);
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
     std::vector<Tensor> output;
     HelperInterpNearest::init_indices_weights(
       scalar_type, output, output_size, ndims, reshape_dim, HelperInterpNearest::interp_size);
@@ -1175,9 +1168,8 @@ struct HelperInterpLinear : public HelperInterpBase {
   static inline std::vector<Tensor> compute_indices_weights(
     at::ScalarType scalar_type,
     int64_t input_size, int64_t output_size, int64_t stride, int64_t ndims, int64_t reshape_dim,
-    bool align_corners, const std::optional<double> opt_scale
+    bool align_corners, const std::optional<double>& opt_scale
   ) {
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
     std::vector<Tensor> output;
     HelperInterpLinear::init_indices_weights(
       scalar_type, output, output_size, ndims, reshape_dim, HelperInterpLinear::interp_size);
@@ -1230,7 +1222,7 @@ struct HelperInterpLinear : public HelperInterpBase {
     int64_t ndims,
     int64_t reshape_dim,
     bool align_corners,
-    const std::optional<double> opt_scale,
+    const std::optional<double>& opt_scale,
     bool antialias
   ) {
 
@@ -1266,7 +1258,7 @@ struct HelperInterpLinear : public HelperInterpBase {
     int64_t ndims,
     int64_t reshape_dim,
     bool align_corners,
-    const std::optional<double> opt_scale,
+    const std::optional<double>& opt_scale,
     bool antialias,
     bool align_i32=false
   ) {
@@ -1296,9 +1288,8 @@ struct HelperInterpCubic : public HelperInterpBase {
   static inline std::vector<Tensor> compute_indices_weights(
     at::ScalarType scalar_type,
     int64_t input_size, int64_t output_size, int64_t stride, int64_t ndims, int64_t reshape_dim,
-    bool align_corners, const std::optional<double> opt_scale
+    bool align_corners, const std::optional<double>& opt_scale
   ) {
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
     std::vector<Tensor> output;
     HelperInterpCubic::init_indices_weights(
       scalar_type, output, output_size, ndims, reshape_dim, HelperInterpCubic::interp_size);
@@ -1364,7 +1355,7 @@ struct HelperInterpCubic : public HelperInterpBase {
     int64_t ndims,
     int64_t reshape_dim,
     bool align_corners,
-    const std::optional<double> opt_scale,
+    const std::optional<double>& opt_scale,
     bool antialias
   ) {
 
@@ -1400,7 +1391,7 @@ struct HelperInterpCubic : public HelperInterpBase {
     int64_t ndims,
     int64_t reshape_dim,
     bool align_corners,
-    const std::optional<double> opt_scale,
+    const std::optional<double>& opt_scale,
     bool antialias,
     bool align_i32=false
   ) {
@@ -1448,8 +1439,6 @@ void upsample_generic_Nd_kernel_impl(
   }
   auto restrided_input = input.as_strided(shape, strides);
 
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  std::vector<std::vector<Tensor>> indices_weights;
 
   constexpr int interp_size = F::interp_size;
   auto input_scalar_type = input.scalar_type();
@@ -1459,8 +1448,9 @@ void upsample_generic_Nd_kernel_impl(
     input_scalar_type = at::ScalarType::Float;
   }
 
+  std::vector<std::vector<Tensor>> indices_weights;
+  indices_weights.reserve(out_ndims);
   for (const auto i : c10::irange(out_ndims)) {
-    // NOLINTNEXTLINE(performance-inefficient-vector-operation)
     indices_weights.emplace_back(
       F::compute_indices_weights(
         input_scalar_type, input.size(i + 2), oshape[i + 2],
@@ -1505,32 +1495,16 @@ template <typename scalar_t, bool is_horizontal>
 void cpu_upsample_generic_aa(at::TensorIterator& iter, unsigned int weights_precision) {
 
   auto loop = [&](char** data, const int64_t* strides, int64_t n) {
-    if (is_horizontal) {
+    if constexpr (is_horizontal) {
 
       // Strides are : X 0 | 8 8 8 0 8  (Channels first)
       // Strides are : X X | 0 0 0 0 0  (Channels last)
-      // upsampling data within a contiguous dimension (aka horizontal resampling)
-      if ((strides[0] == sizeof(scalar_t)) && (strides[1] == sizeof(scalar_t)) &&
-          is_zero_stride<3 + 2>(&strides[2])) {
-        // channels last case
-        basic_loop_aa_horizontal<scalar_t>(
-            data, strides, n, weights_precision);
-      } else {
-        basic_loop_aa_horizontal<scalar_t>(
-            data, strides, n, weights_precision);
-      }
+      basic_loop_aa_horizontal<scalar_t>(data, strides, n, weights_precision);
     } else {
       // Strides are : X Y | 0 0 0 0 0 (Channels first)
       // Strides are : X X | 0 0 0 0 0 (Channels last)
       // upsampling data between contiguous dimensions (aka vertical resampling)
-      if ((strides[0] == sizeof(scalar_t)) && (strides[1] == sizeof(scalar_t)) &&
-          is_zero_stride<3 + 2>(&strides[2])) {
-        basic_loop_aa_vertical<scalar_t>(
-            data, strides, n, weights_precision);
-      } else {
-        basic_loop_aa_vertical<scalar_t>(
-            data, strides, n, weights_precision);
-      }
+      basic_loop_aa_vertical<scalar_t>(data, strides, n, weights_precision);
     }
   };
 
@@ -1565,11 +1539,11 @@ void _separable_upsample_generic_Nd_kernel_impl_single_dim(
 
   std::vector<Tensor> indices_weights;
   unsigned int weights_precision = 0;
-  int unused;
 
   if (input_scalar_type == at::kByte) {
     // This is a special branch to provide uint8 dtype support for bilinear and bicubic modes only
     TORCH_INTERNAL_ASSERT(F::interp_size == 2 || F::interp_size == 4);
+    int unused = 0;
     std::tie(indices_weights, unused, weights_precision) =
       F::compute_index_ranges_int16_weights(
         input.size(interp_dim), oshape[interp_dim],
@@ -1998,9 +1972,8 @@ void cpu_upsample_genNd_backward_aa(
     std::vector<scalar_t> wx(interp_width, 0.0);
     std::vector<scalar_t> wy(interp_height, 0.0);
 
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    int64_t xmin, ymin;
-    int64_t xsize, ysize;
+    int64_t xmin = 0, ymin = 0;
+    int64_t xsize = 0, ysize = 0;
 
     typedef scalar_t (*aa_filter_fn_t)(scalar_t);
     aa_filter_fn_t filter_fn = &F::aa_filter;
