@@ -12,7 +12,9 @@
 #else
 #include <ATen/ops/_to_dense_native.h>
 #include <ATen/ops/empty.h>
+#include <ATen/ops/empty_like.h>
 #include <ATen/ops/empty_native.h>
+#include <ATen/ops/from_blob.h>
 #include <ATen/ops/mkldnn_reorder_conv2d_weight_native.h>
 #include <ATen/ops/mkldnn_reorder_conv3d_weight_native.h>
 #include <ATen/ops/to_mkldnn_native.h>
@@ -24,20 +26,22 @@ namespace at { namespace native {
 
 #if AT_MKLDNN_ENABLED()
 
-Tensor mkldnn_to_dense(const Tensor& mkldnn_tensor, c10::optional<ScalarType> dtype, c10::optional<bool> masked_grad) {
+Tensor mkldnn_to_dense(const Tensor& mkldnn_tensor, std::optional<ScalarType> dtype, std::optional<bool> masked_grad) {
   TORCH_CHECK(mkldnn_tensor.scalar_type() == ScalarType::Float ||
               mkldnn_tensor.scalar_type() == ScalarType::BFloat16 ||
+              mkldnn_tensor.scalar_type() == ScalarType::Half ||
               mkldnn_tensor.scalar_type() == ScalarType::Byte ||
               mkldnn_tensor.scalar_type() == ScalarType::Char,
-              "mkldnn_to_dense expects float, bfloat16, uint8, int8 tensor input");
+              "mkldnn_to_dense expects float, bfloat16, half, uint8, int8 tensor input");
   ideep::tensor& stensor = itensor_from_mkldnn(mkldnn_tensor);
   auto dims = stensor.get_dims();
   auto data_type = dtype.has_value() ? dtype.value() : mkldnn_tensor.scalar_type();
   TORCH_CHECK(data_type == ScalarType::Float ||
               data_type == ScalarType::BFloat16 ||
+              data_type == ScalarType::Half ||
               data_type == ScalarType::Byte ||
               data_type == ScalarType::Char,
-              "mkldnn tensor only can be converted to be a float, bfloat16, uint8, int8 cpu tensor")
+              "mkldnn tensor only can be converted to be a float, bfloat16, Half, uint8, int8 cpu tensor")
   if (mkldnn_tensor.scalar_type() == ScalarType::Byte || mkldnn_tensor.scalar_type() == ScalarType::Char) {
     // For int8, uint8 input, we should not change the data type.
     TORCH_CHECK(mkldnn_tensor.scalar_type() == data_type,
@@ -55,28 +59,33 @@ Tensor mkldnn_to_dense(const Tensor& mkldnn_tensor, c10::optional<ScalarType> dt
       : (data_type == ScalarType::BFloat16
          ? stensor.to_public(cpu_tensor.template data_ptr<BFloat16>(),
                          ideep::tensor::data_type::bf16)
-         : (data_type == ScalarType::Byte
-            ? stensor.to_public(cpu_tensor.template data_ptr<uint8_t>(),
-                            ideep::tensor::data_type::u8)
-            : stensor.to_public(cpu_tensor.template data_ptr<int8_t>(),
-                            ideep::tensor::data_type::s8)
-         )
+         : (data_type == ScalarType::Half
+            ? stensor.to_public(cpu_tensor.template data_ptr<Half>(),
+                            ideep::tensor::data_type::f16)
+          : (data_type == ScalarType::Byte
+              ? stensor.to_public(cpu_tensor.template data_ptr<uint8_t>(),
+                              ideep::tensor::data_type::u8)
+              : stensor.to_public(cpu_tensor.template data_ptr<int8_t>(),
+                              ideep::tensor::data_type::s8)
+            )
+           )
       );
   cpu_tensor.as_strided_(dims, pub_tensor.get_strides());
   // Make sure that NC11 strides follow formula of contiguous tensor.
   return cpu_tensor.contiguous().resize_(dims, c10::MemoryFormat::Contiguous);
 }
 
-Tensor dense_to_mkldnn(const Tensor& cpu_tensor, c10::optional<ScalarType> dtype) {
+Tensor dense_to_mkldnn(const Tensor& cpu_tensor, std::optional<ScalarType> dtype) {
   TORCH_CHECK(cpu_tensor.device().is_cpu(),
              "dense_to_mkldnn expects CPU tensor input");
   TORCH_CHECK(cpu_tensor.layout() == Layout::Strided,
              "dense_to_mkldnn expects strided tensor input");
   TORCH_CHECK(cpu_tensor.scalar_type() == ScalarType::Float ||
               cpu_tensor.scalar_type() == ScalarType::BFloat16 ||
+              cpu_tensor.scalar_type() == ScalarType::Half ||
               cpu_tensor.scalar_type() == ScalarType::Byte ||
               cpu_tensor.scalar_type() == ScalarType::Char,
-             "dense_to_mkldnn expects float, bfloat16, uint8, int8 tensor input");
+             "dense_to_mkldnn expects float, bfloat16, half, uint8, int8 tensor input");
   TORCH_CHECK(cpu_tensor.dim() <= 5,
              "Can't convert cpu tensor with the number of dimensions > 5");
   // NOTE: forbid direct convert from non-contiguous (or channels last) to `ideep::tensor`.
@@ -89,9 +98,10 @@ Tensor dense_to_mkldnn(const Tensor& cpu_tensor, c10::optional<ScalarType> dtype
   }
   TORCH_CHECK(data_type == ScalarType::Float ||
               data_type == ScalarType::BFloat16 ||
+              data_type == ScalarType::Half ||
               data_type == ScalarType::Byte ||
               data_type == ScalarType::Char,
-              "cpu tensor only can be converted to be a float, bfloat16, uint8, int8 mkldnn tensor")
+              "cpu tensor only can be converted to be a float, bfloat16, half, uint8, int8 mkldnn tensor")
   Tensor mkldnn_tensor = empty_mkldnn(cpu_tensor_cont.sizes(), data_type,
                                       cpu_tensor_cont.options().layout_opt(), cpu_tensor_cont.options().device_opt(),
                                       cpu_tensor_cont.options().pinned_memory_opt());
@@ -104,6 +114,10 @@ Tensor dense_to_mkldnn(const Tensor& cpu_tensor, c10::optional<ScalarType> dtype
     dtensor.feed_from(dtensor.get_dims(),
                       ideep::tensor::data_type::bf16,
                       cpu_tensor_cont.template data_ptr<BFloat16>());
+  } else if (cpu_tensor.scalar_type() == ScalarType::Half) {
+    dtensor.feed_from(dtensor.get_dims(),
+                      ideep::tensor::data_type::f16,
+                      cpu_tensor_cont.template data_ptr<Half>());
   } else if (cpu_tensor.scalar_type() == ScalarType::Byte) {
     dtensor.feed_from(dtensor.get_dims(),
                       ideep::tensor::data_type::u8,
@@ -131,10 +145,7 @@ Tensor mkldnn_reorder_conv2d_weight(
     IntArrayRef dilation,
     int64_t groups,
     c10::OptionalArrayRef<int64_t> input_size) {
-  if (self.scalar_type() == ScalarType::BFloat16) {
-    TORCH_CHECK(mkldnn_bf16_device_check(),
-        "mkldnn_reorder_conv2d_weight: bf16 path needs the cpu support avx512bw, avx512vl and avx512dq");
-  }
+  mkldnn_check_low_precision(self.scalar_type(), "mkldnn_reorder_conv2d_weight");
   const auto padding_expanded = expand_param_if_needed(padding, "padding", 2);
   const auto stride_expanded = expand_param_if_needed(stride, "stride", 2);
   const auto dilation_expanded = expand_param_if_needed(dilation, "dilation", 2);
@@ -189,26 +200,40 @@ Tensor mkldnn_reorder_conv3d_weight(
     IntArrayRef padding,
     IntArrayRef stride,
     IntArrayRef dilation,
-    int64_t groups) {
-  if (self.scalar_type() == ScalarType::BFloat16) {
-    TORCH_CHECK(mkldnn_bf16_device_check(),
-        "mkldnn_reorder_conv3d_weight: bf16 path needs the cpu support avx512bw, avx512vl and avx512dq");
-  }
+    int64_t groups,
+    c10::OptionalArrayRef<int64_t> input_size) {
+  mkldnn_check_low_precision(self.scalar_type(), "mkldnn_reorder_conv3d_weight");
   const auto padding_expanded = expand_param_if_needed(padding, "padding", 3);
   const auto stride_expanded = expand_param_if_needed(stride, "stride", 3);
   const auto dilation_expanded = expand_param_if_needed(dilation, "dilation", 3);
-  auto w = itensor_from_mkldnn(self);
 
-  auto desc =
-      ideep::convolution_forward::expected_weights_desc(
-          w.get_dims(),
-          w.get_data_type(),
-          stride_expanded,
-          padding_expanded,
-          padding_expanded,
-          dilation_expanded,
-          groups,
-          ideep::algorithm::convolution_direct);
+  ideep::dims src_dims = ideep::dims();
+  bool is_channels_last = false;
+  auto memory_format = at::MemoryFormat::Contiguous;
+  if (input_size.has_value()) {
+    src_dims = input_size.value().vec();
+    // if has input size, we always use channels last.
+    is_channels_last = true;
+    memory_format = at::MemoryFormat::ChannelsLast3d;
+  }
+
+  auto self_ = self.is_mkldnn() ? self : self.contiguous(memory_format);
+  auto w = itensor_from_tensor(self_);
+
+  auto desc = ideep::convolution_forward::expected_weights_desc(
+      w.get_dims(),
+      w.get_data_type(),
+      stride_expanded,
+      padding_expanded,
+      padding_expanded,
+      dilation_expanded,
+      groups,
+      ideep::algorithm::convolution_direct,
+      ideep::prop_kind::forward,
+      w.get_data_type(),
+      src_dims,
+      ideep::attr_t(),
+      is_channels_last);
   ideep::tensor result;
   result.init(desc);
   result.feed_from(w);
@@ -216,13 +241,25 @@ Tensor mkldnn_reorder_conv3d_weight(
   return new_with_itensor_mkldnn(std::move(result), optTypeMetaToScalarType(self.options().dtype_opt()), self.options().device_opt());
 }
 
+static Tensor mkldnn_reorder_conv_weight(
+    const Tensor& self,
+    IntArrayRef padding,
+    IntArrayRef stride,
+    IntArrayRef dilation,
+    int64_t groups,
+    c10::OptionalArrayRef<int64_t> input_size) {
+  TORCH_CHECK((self.dim() == 4 || self.dim() == 5), "mkldnn_reorder_conv_weight only supports conv2d and conv3d");
+  if (self.dim() == 4) {
+    return at::native::mkldnn_reorder_conv2d_weight(self, padding, stride, dilation, groups, input_size);
+  } else {
+    return at::native::mkldnn_reorder_conv3d_weight(self, padding, stride, dilation, groups, input_size);
+  }
+}
+
 static Tensor mkldnn_reorder_linear_weight(
     const Tensor& self,
-    c10::optional<int64_t> batch_size_opt) {
-  if (self.scalar_type() == ScalarType::BFloat16) {
-    TORCH_CHECK(mkldnn_bf16_device_check(),
-        "mkldnn_reorder_linear_weight: bf16 path needs the cpu support avx512bw, avx512vl and avx512dq");
-  }
+    std::optional<int64_t> batch_size_opt) {
+  mkldnn_check_low_precision(self.scalar_type(), "mkldnn_reorder_linear_weight");
   auto out_features = self.size(0);
   auto in_features = self.size(1);
   auto self_ = self.contiguous();
@@ -282,7 +319,7 @@ static ideep::tensor::desc get_conv_transpose_expected_weights_desc(
   }
 }
 
-static Tensor mkldnn_reorder_conv_transpose2d_weight(
+static Tensor mkldnn_reorder_conv_transpose_weight(
     const Tensor& self,
     IntArrayRef padding,
     IntArrayRef output_padding,
@@ -290,15 +327,20 @@ static Tensor mkldnn_reorder_conv_transpose2d_weight(
     IntArrayRef dilation,
     int64_t groups,
     c10::OptionalArrayRef<int64_t> input_size) {
+  TORCH_CHECK(
+      (self.dim() == 4 || self.dim() == 5),
+      "mkldnn_reorder_conv_transpose_weight only supports conv_transpose2d and conv_transpose3d");
   c10::impl::ExcludeDispatchKeyGuard edkg(c10::autograd_dispatch_keyset);
-  if (self.scalar_type() == ScalarType::BFloat16) {
-    TORCH_CHECK(mkldnn_bf16_device_check(),
-        "mkldnn_reorder_conv2d_weight: bf16 path needs the cpu support avx512bw, avx512vl and avx512dq");
-  }
-  const auto padding_expanded = expand_param_if_needed(padding, "padding", 2);
-  const auto stride_expanded = expand_param_if_needed(stride, "stride", 2);
-  const auto dilation_expanded = expand_param_if_needed(dilation, "dilation", 2);
-  const auto output_padding_expanded = expand_param_if_needed(output_padding, "output_padding", 2);
+  mkldnn_check_low_precision(
+      self.scalar_type(), "mkldnn_reorder_conv_transpose_weight");
+  int64_t pdim = self.dim() - 2;
+  const auto padding_expanded =
+      expand_param_if_needed(padding, "padding", pdim);
+  const auto stride_expanded = expand_param_if_needed(stride, "stride", pdim);
+  const auto dilation_expanded =
+      expand_param_if_needed(dilation, "dilation", pdim);
+  const auto output_padding_expanded =
+      expand_param_if_needed(output_padding, "output_padding", pdim);
 
   ideep::dims src_dims = ideep::dims();
   bool is_channels_last = false;
@@ -307,7 +349,8 @@ static Tensor mkldnn_reorder_conv_transpose2d_weight(
     src_dims = input_size.value().vec();
     // if has input size, we always use channels last.
     is_channels_last = true;
-    memory_format = at::MemoryFormat::ChannelsLast;
+    memory_format = self.dim() == 4 ? at::MemoryFormat::ChannelsLast
+                                    : at::MemoryFormat::ChannelsLast3d;
   }
 
   auto self_ = self.contiguous(memory_format);
@@ -388,9 +431,7 @@ static std::tuple<ideep::tensor, ideep::tensor> get_lstm_packed_weights(
         get_mkldnn_dtype(weight_hh.scalar_type()),
         ideep::format_tag::ldgoi});
 
-  ideep::tensor::desc packed_desc_ih, packed_desc_hh;
-
-  std::tie(packed_desc_ih, packed_desc_hh) =
+  auto [packed_desc_ih, packed_desc_hh] =
       ideep::lstm_forward_inference::expected_weights_desc(
           output_sizes,
           src_layer,
@@ -442,12 +483,11 @@ static std::vector<Tensor> mkldnn_reorder_mkldnn_rnn_layer_weight(
     batch_size = 10;
   }
 
-  ideep::tensor w1_, w2_;
   at::Tensor packed_w1, packed_w2;
 
   int64_t feature_size = weight0.size(-1);
 
-  std::tie(w1_, w2_) = get_lstm_packed_weights(
+  auto [w1_, w2_] = get_lstm_packed_weights(
     weight0,
     weight1,
     at::zeros(
@@ -479,28 +519,53 @@ static std::vector<Tensor> mkldnn_reorder_mkldnn_rnn_layer_weight(
   return {packed_w1, packed_w2};
 }
 
+static Tensor get_mkldnn_serialized_md(const Tensor& self) {
+  const ideep::tensor packed_w = itensor_from_tensor(self);
+  auto packed_w_desc = packed_w.get_desc();
+  std::vector<uint8_t> serialized_wei_desc;
+
+#if IDEEP_PREREQ(3, 4, 1, 2)
+  serialized_wei_desc = packed_w_desc.get_blob();
+#else
+      TORCH_CHECK(false, "Unexpected IDeep version to do weight serialization.");
+#endif
+  Tensor serialized_md = at::from_blob((void*)serialized_wei_desc.data(), {(int64_t)serialized_wei_desc.size()}, at::TensorOptions(at::kByte));
+  auto res = at::empty_like(serialized_md);
+  // serialized_md shares the buffer with serialized_wei_desc,
+  // which will be released outside of this function thus invalidating the buffer of serialized_md.
+  // A copy is needed here so that res has its own buffer, which remains valid even after serialized_wei_desc is released.
+  res.copy_(serialized_md);
+  return res;
+}
+
 TORCH_LIBRARY_IMPL(mkldnn, CPU, m) {
   m.impl(
       TORCH_SELECTIVE_NAME("mkldnn::_reorder_convolution_transpose_weight"),
-      TORCH_FN(mkldnn_reorder_conv_transpose2d_weight));
+      TORCH_FN(mkldnn_reorder_conv_transpose_weight));
   m.impl(
       TORCH_SELECTIVE_NAME("mkldnn::_reorder_linear_weight"),
       TORCH_FN(mkldnn_reorder_linear_weight));
   m.impl(
       TORCH_SELECTIVE_NAME("mkldnn::_reorder_convolution_weight"),
-      TORCH_FN(mkldnn_reorder_conv2d_weight));
+      TORCH_FN(mkldnn_reorder_conv_weight));
   m.impl(
       TORCH_SELECTIVE_NAME("mkldnn::_reorder_mkldnn_rnn_layer_weight"),
       TORCH_FN(mkldnn_reorder_mkldnn_rnn_layer_weight));
 }
 
+TORCH_LIBRARY_IMPL(mkldnn, MkldnnCPU, m) {
+  m.impl(
+      TORCH_SELECTIVE_NAME("mkldnn::_get_mkldnn_serialized_md"),
+      TORCH_FN(get_mkldnn_serialized_md ));
+}
+
 #else
 
-Tensor mkldnn_to_dense(const Tensor& mkldnn_tensor, c10::optional<ScalarType> dtype, c10::optional<bool> masked_grad) {
+Tensor mkldnn_to_dense(const Tensor& mkldnn_tensor, std::optional<ScalarType> dtype, std::optional<bool> masked_grad) {
   TORCH_CHECK(false, "MKL-DNN build is disabled");
 }
 
-Tensor dense_to_mkldnn(const Tensor& cpu_tensor, c10::optional<ScalarType> dtype) {
+Tensor dense_to_mkldnn(const Tensor& cpu_tensor, std::optional<ScalarType> dtype) {
   TORCH_CHECK(false, "MKL-DNN build is disabled");
 }
 
@@ -519,7 +584,8 @@ Tensor mkldnn_reorder_conv3d_weight(
     IntArrayRef padding,
     IntArrayRef stride,
     IntArrayRef dilation,
-    int64_t groups) {
+    int64_t groups,
+    c10::OptionalArrayRef<int64_t> input_size) {
   TORCH_CHECK(false, "mkldnn_reorder_conv3d_weight: MKL-DNN build is disabled");
 }
 
