@@ -7,9 +7,15 @@ import torch
 import torch.ao.quantization as tq
 from torch import nn
 from torch.ao import pruning
-from torch.testing._internal.common_utils import TestCase
-from torch.ao.quantization.quantize_fx import prepare_fx, convert_fx, convert_to_reference_fx, prepare_qat_fx
 from torch.ao.pruning import fqn_to_module
+from torch.ao.quantization.quantize_fx import (
+    convert_fx,
+    convert_to_reference_fx,
+    prepare_fx,
+    prepare_qat_fx,
+)
+from torch.testing._internal.common_utils import TestCase
+
 
 logging.basicConfig(
     format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
@@ -21,6 +27,7 @@ sparse_defaults = {
     "zeros_per_block": 4,
 }
 
+
 def _get_model_and_sparsifier_and_sparse_config(qconfig=None):
     model = nn.Sequential(
         nn.Linear(4, 4),  # 0
@@ -40,7 +47,7 @@ def _get_model_and_sparsifier_and_sparse_config(qconfig=None):
 
     sparse_config = [
         {
-            "tensor_fqn": '5.weight',
+            "tensor_fqn": "5.weight",
             "sparsity_level": 0.7,
             "sparse_block_shape": (1, 4),
             "zeros_per_block": 4,
@@ -49,15 +56,18 @@ def _get_model_and_sparsifier_and_sparse_config(qconfig=None):
     ]
     return model, sparsifier, sparse_config
 
+
 def _squash_mask_calibrate_and_convert(model, sparsifier, input):
     sparsifier.step()
     sparsifier.squash_mask()
     model(input)
     tq.convert(model, inplace=True)
 
+
 def _calculate_sparsity(tensor):
     return ((tensor == 0).sum() / tensor.numel()).item()
 
+
 # This series of tests are to check the composability goals for sparsity and quantization. Namely
 # that performing quantization and sparsity model manipulations in various orderings
 # does not cause problems
@@ -70,7 +80,9 @@ class TestComposability(TestCase):
             mod,
             sparsifier,
             sparse_config,
-        ) = _get_model_and_sparsifier_and_sparse_config(tq.get_default_qconfig("fbgemm"))
+        ) = _get_model_and_sparsifier_and_sparse_config(
+            tq.get_default_qconfig("fbgemm")
+        )
 
         tq.prepare(mod, inplace=True)
         sparsifier.prepare(mod, config=sparse_config)
@@ -81,9 +93,7 @@ class TestComposability(TestCase):
         # check that correct observers were inserted
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
 
-        _squash_mask_calibrate_and_convert(
-            mod, sparsifier, torch.randn(1, 4, 4, 4)
-        )
+        _squash_mask_calibrate_and_convert(mod, sparsifier, torch.randn(1, 4, 4, 4))
 
         # check that final module is the expected quantized module and that the model runs
         self.assertTrue(isinstance(mod[5], torch.ao.nn.quantized.Linear))
@@ -99,7 +109,9 @@ class TestComposability(TestCase):
             mod,
             sparsifier,
             sparse_config,
-        ) = _get_model_and_sparsifier_and_sparse_config(tq.get_default_qconfig("fbgemm"))
+        ) = _get_model_and_sparsifier_and_sparse_config(
+            tq.get_default_qconfig("fbgemm")
+        )
 
         sparsifier.prepare(mod, config=sparse_config)
         tq.prepare(mod, inplace=True)
@@ -110,12 +122,10 @@ class TestComposability(TestCase):
         self.assertTrue(hasattr(mod[5], "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
 
-        _squash_mask_calibrate_and_convert(
-            mod, sparsifier, torch.randn(1, 4, 4, 4)
-        )
+        _squash_mask_calibrate_and_convert(mod, sparsifier, torch.randn(1, 4, 4, 4))
 
         # check that final module is the expected quantized module and that the model runs
         self.assertTrue(isinstance(mod[5], torch.ao.nn.quantized.Linear))
@@ -130,7 +140,9 @@ class TestComposability(TestCase):
             mod,
             sparsifier,
             sparse_config,
-        ) = _get_model_and_sparsifier_and_sparse_config(tq.get_default_qconfig("fbgemm"))
+        ) = _get_model_and_sparsifier_and_sparse_config(
+            tq.get_default_qconfig("fbgemm")
+        )
 
         sparsifier.prepare(mod, config=sparse_config)
         tq.prepare(mod, inplace=True)
@@ -141,7 +153,7 @@ class TestComposability(TestCase):
         self.assertTrue(hasattr(mod[5], "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(mod[5].weight)
@@ -168,7 +180,9 @@ class TestComposability(TestCase):
             mod,
             sparsifier,
             sparse_config,
-        ) = _get_model_and_sparsifier_and_sparse_config(tq.get_default_qconfig("fbgemm"))
+        ) = _get_model_and_sparsifier_and_sparse_config(
+            tq.get_default_qconfig("fbgemm")
+        )
         sparsifier.prepare(mod, config=sparse_config)
         tq.fuse_modules(mod, [["5", "6"]], inplace=True)
         mod[5].qconfig = tq.get_default_qconfig("fbgemm")
@@ -180,11 +194,9 @@ class TestComposability(TestCase):
         self.assertTrue(hasattr(mod[5][0], "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
-        _squash_mask_calibrate_and_convert(
-            mod, sparsifier, torch.randn(1, 4, 4, 4)
-        )
+        _squash_mask_calibrate_and_convert(mod, sparsifier, torch.randn(1, 4, 4, 4))
 
         # check that final module is the expected quantized module and that the model runs
         self.assertTrue(isinstance(mod[5], torch.ao.nn.intrinsic.quantized.LinearReLU))
@@ -197,7 +209,9 @@ class TestComposability(TestCase):
             mod,
             sparsifier,
             _,
-        ) = _get_model_and_sparsifier_and_sparse_config(tq.get_default_qconfig("fbgemm"))
+        ) = _get_model_and_sparsifier_and_sparse_config(
+            tq.get_default_qconfig("fbgemm")
+        )
         tq.fuse_modules(mod, [["5", "6"]], inplace=True)
 
         # its absolutely broken by fusion but will still work if you put the correct fqn in
@@ -221,7 +235,7 @@ class TestComposability(TestCase):
         self.assertTrue(hasattr(mod[5][0], "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(mod[5][0].weight)
@@ -242,7 +256,7 @@ class TestComposability(TestCase):
 
     # This tests whether performing sparse prepare before qat prepare causes issues.
     # The primary worries were that qat_prep wouldn't recognize the parametrized
-    # modules and that the convert step for qat would remove the paramerizations
+    # modules and that the convert step for qat would remove the parametrizations
     # from the modules.
     def test_s_prep_before_qat_prep(self):
         (
@@ -258,12 +272,10 @@ class TestComposability(TestCase):
         self.assertTrue(hasattr(mod[5], "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
         self.assertTrue(isinstance(mod[5], torch.ao.nn.qat.Linear))
-        _squash_mask_calibrate_and_convert(
-            mod, sparsifier, torch.randn(1, 4, 4, 4)
-        )
+        _squash_mask_calibrate_and_convert(mod, sparsifier, torch.randn(1, 4, 4, 4))
         # check that final module is the expected quantized module and that the model runs
         self.assertTrue(isinstance(mod[5], torch.ao.nn.quantized.Linear))
         self.assertEqual(mod(torch.randn(1, 4, 4, 4)).shape, torch.Size([1, 4, 4, 4]))
@@ -297,13 +309,11 @@ class TestComposability(TestCase):
         self.assertTrue(hasattr(mod[5], "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(hasattr(mod[5], "activation_post_process"))
         self.assertTrue(isinstance(mod[5], torch.ao.nn.qat.Linear))
 
-        _squash_mask_calibrate_and_convert(
-            mod, sparsifier, torch.randn(1, 4, 4, 4)
-        )
+        _squash_mask_calibrate_and_convert(mod, sparsifier, torch.randn(1, 4, 4, 4))
 
         # check that final module is the expected quantized module and that the model runs
         self.assertTrue(isinstance(mod[5], torch.ao.nn.quantized.Linear))
@@ -313,6 +323,7 @@ class TestComposability(TestCase):
         cur_sparsity = _calculate_sparsity(mod[5]._weight_bias()[0])
         self.assertGreaterAlmostEqual(cur_sparsity, sparse_config[0]["sparsity_level"])
 
+
 def _module_has_activation_post_process(model, fqn_of_module):
     for node in model.graph.nodes:
         # look for an observer whose arg is the target module
@@ -321,10 +332,12 @@ def _module_has_activation_post_process(model, fqn_of_module):
                 return True
     return False
 
+
 class TestFxComposability(TestCase):
     r"""This series of tests checks that various steps of the quantization and sparsity flow
     compose cleanly despite variation in sequencing.
     """
+
     def test_q_prep_fx_before_s_prep(self):
         r"""
         This test checks that the ordering of prepare_fx -> sparse prepare -> convert_fx
@@ -340,10 +353,11 @@ class TestFxComposability(TestCase):
 
         example = torch.randn(1, 4, 4, 4)
         qconfig = tq.get_default_qconfig("fbgemm")
-        qconfig_mapping = tq.QConfigMapping() \
-            .set_module_name("4", qconfig) \
+        qconfig_mapping = (
+            tq.QConfigMapping()
+            .set_module_name("4", qconfig)
             .set_module_name("5", qconfig)
-
+        )
 
         mod = prepare_fx(mod, qconfig_mapping, (example,))
 
@@ -366,7 +380,7 @@ class TestFxComposability(TestCase):
         self.assertTrue(hasattr(fqn_to_module(mod, "5.0"), "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(_module_has_activation_post_process(mod, "5"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(fqn_to_module(mod, "5.0.weight"))
@@ -374,7 +388,11 @@ class TestFxComposability(TestCase):
         mod = convert_fx(mod)
 
         # check that final module is the expected quantized module and that the model runs
-        self.assertTrue(isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.quantized.LinearReLU))
+        self.assertTrue(
+            isinstance(
+                fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.quantized.LinearReLU
+            )
+        )
         self.assertEqual(mod(example).shape, torch.Size([1, 4, 4, 4]))
 
         # check that module was actually sparsified
@@ -399,9 +417,11 @@ class TestFxComposability(TestCase):
 
         example = torch.randn(1, 4, 4, 4)
         qconfig = tq.get_default_qconfig("fbgemm")
-        qconfig_mapping = tq.QConfigMapping() \
-            .set_module_name("4", qconfig) \
+        qconfig_mapping = (
+            tq.QConfigMapping()
+            .set_module_name("4", qconfig)
             .set_module_name("5", qconfig)
+        )
 
         mod = prepare_fx(mod, qconfig_mapping, (example,))
 
@@ -424,7 +444,7 @@ class TestFxComposability(TestCase):
         self.assertTrue(hasattr(fqn_to_module(mod, "5.0"), "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(_module_has_activation_post_process(mod, "5"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(fqn_to_module(mod, "5.0.weight"))
@@ -432,9 +452,15 @@ class TestFxComposability(TestCase):
         mod = convert_to_reference_fx(mod)
 
         # check that final module is the expected quantized module and that the model runs
-        self.assertTrue(isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.LinearReLU))
+        self.assertTrue(
+            isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.LinearReLU)
+        )
         self.assertEqual(mod(example).shape, torch.Size([1, 4, 4, 4]))
-        self.assertTrue(isinstance(fqn_to_module(mod, "5.0"), torch.ao.nn.quantized.reference.Linear))
+        self.assertTrue(
+            isinstance(
+                fqn_to_module(mod, "5.0"), torch.ao.nn.quantized.reference.Linear
+            )
+        )
 
         # check that module was actually sparsified
         cur_sparsity = _calculate_sparsity(fqn_to_module(mod, "5.0.weight"))
@@ -459,9 +485,11 @@ class TestFxComposability(TestCase):
 
         example = torch.randn(1, 4, 4, 4)
         qconfig = tq.get_default_qconfig("fbgemm")
-        qconfig_mapping = tq.QConfigMapping() \
-            .set_module_name("4", qconfig) \
+        qconfig_mapping = (
+            tq.QConfigMapping()
+            .set_module_name("4", qconfig)
             .set_module_name("5", qconfig)
+        )
         mod = prepare_fx(mod, qconfig_mapping, (example,))
 
         # check that correct modules had parametrizations added and
@@ -470,7 +498,7 @@ class TestFxComposability(TestCase):
         self.assertTrue(hasattr(fqn_to_module(mod, "5.0"), "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(_module_has_activation_post_process(mod, "5"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(fqn_to_module(mod, "5.0.weight"))
@@ -478,7 +506,11 @@ class TestFxComposability(TestCase):
         mod = convert_fx(mod)
 
         # check that final module is the expected quantized module and that the model runs
-        self.assertTrue(isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.quantized.LinearReLU))
+        self.assertTrue(
+            isinstance(
+                fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.quantized.LinearReLU
+            )
+        )
         self.assertEqual(mod(example).shape, torch.Size([1, 4, 4, 4]))
 
         # check that module was actually sparsified
@@ -504,19 +536,23 @@ class TestFxComposability(TestCase):
 
         example = torch.randn(1, 4, 4, 4)
         qconfig = tq.get_default_qat_qconfig("fbgemm")
-        qconfig_mapping = tq.QConfigMapping() \
-            .set_module_name("4", qconfig) \
+        qconfig_mapping = (
+            tq.QConfigMapping()
+            .set_module_name("4", qconfig)
             .set_module_name("5", qconfig)
+        )
         mod = prepare_qat_fx(mod, qconfig_mapping, (example,))
 
         # check that correct modules had parametrizations added and
         # that none were lost during prepare
         self.assertTrue(hasattr(fqn_to_module(mod, "0.0"), "parametrizations"))
         self.assertTrue(hasattr(fqn_to_module(mod, "5"), "parametrizations"))
-        self.assertTrue(isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.qat.LinearReLU))
+        self.assertTrue(
+            isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.qat.LinearReLU)
+        )
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(_module_has_activation_post_process(mod, "5"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(fqn_to_module(mod, "5.weight"))
@@ -524,7 +560,11 @@ class TestFxComposability(TestCase):
         mod = convert_fx(mod)
 
         # check that final module is the expected quantized module and that the model runs
-        self.assertTrue(isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.quantized.LinearReLU))
+        self.assertTrue(
+            isinstance(
+                fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.quantized.LinearReLU
+            )
+        )
         self.assertEqual(mod(example).shape, torch.Size([1, 4, 4, 4]))
 
         # check that module was actually sparsified
@@ -550,9 +590,11 @@ class TestFxComposability(TestCase):
 
         example = torch.randn(1, 4, 4, 4)
         qconfig = tq.get_default_qconfig("fbgemm")
-        qconfig_mapping = tq.QConfigMapping() \
-            .set_module_name("4", qconfig) \
+        qconfig_mapping = (
+            tq.QConfigMapping()
+            .set_module_name("4", qconfig)
             .set_module_name("5", qconfig)
+        )
         mod = prepare_fx(mod, qconfig_mapping, (example,))
 
         # check that correct modules had parametrizations added and
@@ -561,7 +603,7 @@ class TestFxComposability(TestCase):
         self.assertTrue(hasattr(fqn_to_module(mod, "5.0"), "parametrizations"))
 
         # check that correct observers were inserted and that matching
-        # occured successfully
+        # occurred successfully
         self.assertTrue(_module_has_activation_post_process(mod, "5"))
         sparsifier.step()
         sparsity_level = _calculate_sparsity(fqn_to_module(mod, "5.0.weight"))
@@ -569,9 +611,15 @@ class TestFxComposability(TestCase):
         mod = convert_to_reference_fx(mod)
 
         # check that final module is the expected quantized module and that the model runs
-        self.assertTrue(isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.LinearReLU))
+        self.assertTrue(
+            isinstance(fqn_to_module(mod, "5"), torch.ao.nn.intrinsic.LinearReLU)
+        )
         self.assertEqual(mod(example).shape, torch.Size([1, 4, 4, 4]))
-        self.assertTrue(isinstance(fqn_to_module(mod, "5.0"), torch.ao.nn.quantized.reference.Linear))
+        self.assertTrue(
+            isinstance(
+                fqn_to_module(mod, "5.0"), torch.ao.nn.quantized.reference.Linear
+            )
+        )
 
         # check that module was actually sparsified
         cur_sparsity = _calculate_sparsity(fqn_to_module(mod, "5.0.weight"))
