@@ -1,21 +1,29 @@
 # Owner(s): ["module: onnx"]
 from __future__ import annotations
 
+import logging
 import tempfile
+from typing import Mapping, Tuple, TYPE_CHECKING
 
 import onnx
+import onnx.inliner
+
 import pytorch_test_common
-import torch
 import transformers  # type: ignore[import]
+
+import torch
 from torch import nn
 from torch._subclasses import fake_tensor
 from torch.nn import functional as F
 from torch.onnx import dynamo_export, ExportOptions
-from torch.onnx._internal.diagnostics import infra
 from torch.onnx._internal.fx import diagnostics, registration
 from torch.testing._internal import common_utils
 
 
+if TYPE_CHECKING:
+    from torch.onnx._internal.diagnostics import infra
+
+
 def assert_has_diagnostics(
     diagnostic_context: diagnostics.DiagnosticContext,
     rule: infra.Rule,
@@ -91,15 +99,11 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                 diagnostics.rules.find_opschema_matched_symbolic_function,
                 name="optional_inputs",
             ),
-            common_utils.subtest(
-                diagnostics.rules.op_level_debugging,
-                name="get_attr_node_in_op_level_debug",
-            ),
         ],
     )
     def test_mnist_exported_with_no_warnings(self, diagnostic_rule):
         class MNISTModel(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv1 = nn.Conv2d(1, 32, 3, 1, bias=False)
                 self.conv2 = nn.Conv2d(32, 64, 3, 1, bias=False)
@@ -120,12 +124,10 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                 return output
 
         tensor_x = torch.rand((64, 1, 28, 28), dtype=torch.float32)
-        export_output = dynamo_export(
-            MNISTModel(), tensor_x, export_options=ExportOptions(op_level_debug=True)
-        )
+        onnx_program = dynamo_export(MNISTModel(), tensor_x)
 
         assert_has_diagnostics(
-            export_output.diagnostic_context,
+            onnx_program.diagnostic_context,
             diagnostic_rule,
             diagnostics.levels.NONE,
             expected_node="aten.convolution.default",
@@ -156,31 +158,8 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                 return torch.sum(values)
 
         x = torch.arange(1.0, 6.0, requires_grad=True)
-        export_output = dynamo_export(
-            TopKModel(), x, export_options=self.export_options
-        )
 
-    def test_unsupported_indices_fake_tensor_generated_with_op_level_debug(self):
-        class EmbedModelWithoutPaddingIdx(torch.nn.Module):
-            def forward(self, input, emb):
-                return torch.nn.functional.embedding(input, emb)
-
-        model = EmbedModelWithoutPaddingIdx()
-        x = torch.randint(4, (4, 3, 2))
-        embedding_matrix = torch.rand(10, 3)
-
-        export_output = dynamo_export(
-            model,
-            x,
-            embedding_matrix,
-            export_options=ExportOptions(op_level_debug=True),
-        )
-        assert_has_diagnostics(
-            export_output.diagnostic_context,
-            diagnostics.rules.op_level_debugging,
-            diagnostics.levels.WARNING,
-            expected_node="aten.embedding.default",
-        )
+        _ = dynamo_export(TopKModel(), x, export_options=self.export_options)
 
     def test_unsupported_function_schema_raises_diagnostic_warning_when_found_nearest_match(
         self,
@@ -190,10 +169,10 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                 return input.new_zeros(())
 
         x = torch.randn((2, 3), dtype=torch.float32)
-        export_output = dynamo_export(TraceModel(), x)
+        onnx_program = dynamo_export(TraceModel(), x)
 
         assert_has_diagnostics(
-            export_output.diagnostic_context,
+            onnx_program.diagnostic_context,
             diagnostics.rules.find_opschema_matched_symbolic_function,
             diagnostics.levels.WARNING,
             expected_node="aten.new_zeros.default",
@@ -203,7 +182,7 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
         self,
     ):
         class TraceModel(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv2 = torch.nn.Conv2d(
                     16, 33, (3, 5), stride=(2, 1), padding=(4, 2), dilation=(3, 1)
@@ -213,66 +192,132 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                 return self.conv2(input)
 
         x = torch.randn(20, 16, 50, 50)
-        export_output = dynamo_export(
-            TraceModel(), x, export_options=ExportOptions(op_level_debug=False)
-        )
+        onnx_program = dynamo_export(TraceModel(), x)
         assert_has_diagnostics(
-            export_output.diagnostic_context,
+            onnx_program.diagnostic_context,
             diagnostics.rules.find_opschema_matched_symbolic_function,
             diagnostics.levels.NONE,
             expected_node="aten.convolution.default",
         )
 
-    def test_dispatch_overload_fall_back_default_raise_diagnostic_warning(self):
-        class TraceModel(torch.nn.Module):
-            def forward(self, input):
-                return torch.ops.aten.add.Tensor(input, input)
-
-        onnx_registry = torch.onnx.OnnxRegistry()
-        self.assertTrue(
-            onnx_registry.is_registered_op(
-                namespace="aten", op_name="add", overload="Tensor"
-            )
-        )
-        # TODO: Replace this example with a torch custom op when overload is supported
-        # Currently, torch only supports custom op with namespace and op_name
-        aten_add_Tensor = registration.OpName.from_name_parts(
-            namespace="aten", op_name="add", overload="Tensor"
-        )
-        onnx_registry._registry.pop(aten_add_Tensor)
-
-        x = torch.tensor(3)
-        export_output = dynamo_export(
-            TraceModel(), x, export_options=ExportOptions(onnx_registry=onnx_registry)
-        )
-        assert_has_diagnostics(
-            export_output.diagnostic_context,
-            diagnostics.rules.find_operator_overloads_in_onnx_registry,
-            diagnostics.levels.WARNING,
-            expected_node="aten.add.Tensor",
-        )
-
     def test_aten_clone_does_not_raise_warning_of_lack_of_memory_format(self):
         class CustomModule(torch.nn.Module):
             def forward(self, input):
                 return torch.ops.aten.clone(input, memory_format=torch.preserve_format)
 
         x = torch.tensor(3)
-        export_output = dynamo_export(CustomModule(), x)
+        onnx_program = dynamo_export(CustomModule(), x)
         assert_has_diagnostics(
-            export_output.diagnostic_context,
+            onnx_program.diagnostic_context,
             diagnostics.rules.find_opschema_matched_symbolic_function,
             diagnostics.levels.NONE,
             expected_node="aten.clone.default",
         )
 
+    def test_missing_complex_onnx_variant_raises_errors_in_dispatcher(self):
+        registry = torch.onnx.OnnxRegistry()
+
+        # NOTE: simulate unsupported nodes
+        aten_mul_tensor = registration.OpName.from_name_parts(
+            namespace="aten", op_name="mul", overload="Tensor"
+        )
+
+        # Only keep real aten.mul to test missing complex aten.mul
+        registry._registry[aten_mul_tensor] = [
+            onnx_func
+            for onnx_func in registry._registry[aten_mul_tensor]
+            if not onnx_func.is_complex
+        ]
+
+        class TraceModel(torch.nn.Module):
+            def forward(self, input):
+                return torch.ops.aten.mul.Tensor(input, input)
+
+        x = torch.tensor([1 + 2j, 3 + 4j], dtype=torch.complex64)
+
+        with self.assertRaises(torch.onnx.OnnxExporterError) as e:
+            torch.onnx.dynamo_export(
+                TraceModel(),
+                x,
+                export_options=torch.onnx.ExportOptions(onnx_registry=registry),
+            )
+
+    def test_symbolic_shape_of_values_inside_function_is_exported_as_graph_value_info(
+        self,
+    ):
+        class SubModule(torch.nn.Module):
+            def forward(self, x, y, bias):
+                output = x @ y
+                return output + bias
+
+        class Module(torch.nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+                self.submodule = SubModule()
+
+            def forward(self, x, y, bias):
+                return self.submodule(x, y, bias)
+
+        x = torch.randn(2, 3)
+        y = torch.randn(3, 4)
+        bias = torch.randn(4)
+        onnx_program = torch.onnx.dynamo_export(
+            Module(),
+            x,
+            y,
+            bias,
+            export_options=torch.onnx.ExportOptions(dynamic_shapes=True),
+        )
+        model_proto = onnx_program.model_proto
+
+        # Assert value_info for values inside local function can be retrieved
+        def _assert_node_outputs_has_value_info(
+            node: onnx.NodeProto,
+            value_infos: Mapping[str, onnx.ValueInfoProto],
+            local_functions: Mapping[Tuple[str, str], onnx.FunctionProto],
+            exclude_names_in_value_info,
+            function_id: str = "",
+        ):
+            for output in node.output:
+                name = f"{function_id}/{output}" if function_id else output
+                if name not in exclude_names_in_value_info:
+                    self.assertIn(name, value_infos)
+            if node.domain.startswith("pkg.onnxscript.torch_lib"):
+                # No shape info available for values inside torchlib functions.
+                return
+            if (
+                function := local_functions.get((node.domain, node.op_type))
+            ) is not None:
+                for node in function.node:
+                    function_id = f"{function.domain}::{function.name}"
+                    _assert_node_outputs_has_value_info(
+                        node,
+                        value_infos,
+                        local_functions,
+                        exclude_names_in_value_info,
+                        function_id,
+                    )
+
+        type_infos = {vi.name: vi for vi in model_proto.graph.value_info}
+        functions = {(f.domain, f.name): f for f in model_proto.functions}
+        # NOTE: inputs, outputs, and initializers are not included in value_info spec
+        exclude_names_in_value_info = (
+            [input.name for input in model_proto.graph.input]
+            + [output.name for output in model_proto.graph.output]
+            + [init.name for init in model_proto.graph.initializer]
+        )
+        for node in model_proto.graph.node:
+            _assert_node_outputs_has_value_info(
+                node, type_infos, functions, exclude_names_in_value_info
+            )
+
     def test_dynamo_export_retains_readable_parameter_and_buffer_names(self):
         class SubModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv2 = nn.Conv2d(32, 64, 3, 1, bias=False)
                 self.fc1 = nn.Linear(9216, 128, bias=False)
-                self.register_buffer("buffer", torch.randn(1, 128))
+                self.buffer = torch.nn.Buffer(torch.randn(1, 128))
 
             def forward(self, tensor_x: torch.Tensor):
                 tensor_x = self.conv2(tensor_x)
@@ -285,7 +330,7 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                 return tensor_x
 
         class MNISTModel(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv1 = nn.Conv2d(1, 32, 3, 1, bias=False)
                 self.submodule = SubModule()
@@ -302,14 +347,28 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
         tensor_x = torch.rand((64, 1, 28, 28), dtype=torch.float32)
 
         model = MNISTModel()
-        export_output = torch.onnx.dynamo_export(model, tensor_x)
-        model_proto = export_output.model_proto
-        self.assertEqual(
-            {initializer.name for initializer in model_proto.graph.initializer},
-            {*model.state_dict().keys()},
-        )
+        onnx_program = torch.onnx.dynamo_export(model, tensor_x)
+        model_proto = onnx_program.model_proto
 
-    def test_fake_tensor_mode_simple(self):
+        # NOTE: initializers could be optimized away by onnx optimizer
+        onnx_initilizers = {init.name for init in model_proto.graph.initializer}
+        torch_weights = {*model.state_dict().keys()}
+        self.assertTrue(onnx_initilizers.issubset(torch_weights))
+
+    @common_utils.parametrize(
+        "checkpoint_type",
+        [
+            common_utils.subtest(
+                "state_dict",
+                name="state_dict",
+            ),
+            common_utils.subtest(
+                "state_dict",
+                name="checkpoint_file",
+            ),
+        ],
+    )
+    def test_fake_tensor_mode_simple(self, checkpoint_type):
         class Model(torch.nn.Module):
             def __init__(self) -> None:
                 super().__init__()
@@ -323,44 +382,59 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
             x = torch.rand(5, 2, 2)
             model = Model()
             export_options = ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
+            onnx_program = torch.onnx.dynamo_export(
                 model, x, export_options=export_options
             )
 
         assert (
-            export_output is not None
-        ), "ExportOutput must be created on successful export"
+            onnx_program is not None
+        ), "ONNXProgram must be created on successful export"
         assert (
-            export_output.model_proto is not None
+            onnx_program.model_proto is not None
         ), "A model protobuf must be created on a successful export"
-        onnx.checker.check_model(export_output.model_proto, full_check=True)
+        onnx.checker.check_model(onnx_program.model_proto, full_check=True)
         assert (
-            len(export_output.model_proto.graph.initializer) == 0
+            len(onnx_program.model_proto.graph.initializer) == 0
         ), "Initializers cannot exist when fake mode is enabled"
 
-        # Variant 1: Save ONNX proto using Model's state_dict()
-        with tempfile.NamedTemporaryFile(suffix=".onnx") as tmp_onnx_file:
-            model_state_dict = Model().state_dict()  # Create a state_dict for testing
-            export_output.save(tmp_onnx_file.name, model_state_dict=model_state_dict)
-            assert (
-                len(onnx.load(tmp_onnx_file.name).graph.initializer) == 2
-            ), "Initializers must be present after loading it from model_state_dict"
-
-        # Variant 2: Save ONNX proto using Model checkpoint file
-        with tempfile.NamedTemporaryFile(
-            suffix=".onnx"
-        ) as tmp_onnx_file, tempfile.NamedTemporaryFile(
-            suffix=".pt"
-        ) as tmp_checkpoint_file:
-            torch.save(
-                Model().state_dict(), tmp_checkpoint_file.name
-            )  # Create checkpoint file for testing
-            export_output.save(
-                tmp_onnx_file.name, model_state_dict=tmp_checkpoint_file.name
-            )
-            assert (
-                len(onnx.load(tmp_onnx_file.name).graph.initializer) == 2
-            ), "Initializers must be present after loading it from model_state_dict"
+        if checkpoint_type == "state_dict":
+            # Variant 1: Save ONNX proto using Model's state_dict()
+            with tempfile.NamedTemporaryFile(suffix=".onnx") as tmp_onnx_file:
+                model_state_dict = (
+                    Model().state_dict()
+                )  # Create a state_dict for testing
+                onnx_program.save(tmp_onnx_file.name, model_state=model_state_dict)
+                assert (
+                    len(onnx.load(tmp_onnx_file.name).graph.initializer) == 2
+                ), "Initializers must be present after loading it from model_state_dict"
+                # Let's make sure consecutive `save` calls don't create dupes
+                onnx_program.save(tmp_onnx_file.name, model_state=model_state_dict)
+                assert (
+                    len(onnx.load(tmp_onnx_file.name).graph.initializer) == 2
+                ), "Initializers must be present after loading it from model_state_dict"
+        elif checkpoint_type == "checkpoint_file":
+            # Variant 2: Save ONNX proto using Model checkpoint file
+            with tempfile.NamedTemporaryFile(
+                suffix=".onnx"
+            ) as tmp_onnx_file, tempfile.NamedTemporaryFile(
+                suffix=".pt"
+            ) as tmp_checkpoint_file:
+                torch.save(
+                    Model().state_dict(), tmp_checkpoint_file.name
+                )  # Create checkpoint file for testing
+                onnx_program.save(
+                    tmp_onnx_file.name, model_state=tmp_checkpoint_file.name
+                )
+                assert (
+                    len(onnx.load(tmp_onnx_file.name).graph.initializer) == 2
+                ), "Initializers must be present after loading it from model_state_dict"
+                # Let's make sure consecutive `save` calls don't create dupes
+                onnx_program.save(
+                    tmp_onnx_file.name, model_state=tmp_checkpoint_file.name
+                )
+                assert (
+                    len(onnx.load(tmp_onnx_file.name).graph.initializer) == 2
+                ), "Initializers must be present after loading it from model_state_dict"
 
     def test_fake_tensor_mode_simple_invalid_input(self):
         class Model(torch.nn.Module):
@@ -407,150 +481,35 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
                     fake_model, real_x, export_options=export_options
                 )
 
-    # NOTE: To all transformer models, config is preferred to pre-trained model for testing because:
-    # 1. Pre-trained model is too big for CI
-    # 2. Pre-trained model is has uint8/bool issue: https://github.com/huggingface/transformers/issues/21013
-    def test_fake_tensor_mode_huggingface_gpt2(self):
-        config = transformers.GPT2Config(
-            vocab_size=8096, n_positions=256, n_embd=256, n_layer=2, n_head=2
-        )
-        batch, seq = 4, 256
-
-        with torch.onnx.enable_fake_mode() as fake_context:
-            model = transformers.GPT2Model(config).eval()
-            input_ids = torch.randint(0, config.vocab_size, (batch, seq))
-            attention_mask = torch.ones(batch, seq, dtype=torch.bool)
-            position_ids = torch.arange(0, seq, dtype=torch.long)
-            position_ids = position_ids.unsqueeze(0).view(-1, seq)
-
-            export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
-                model,
-                input_ids=input_ids,
-                attention_mask=attention_mask,
-                position_ids=position_ids,
-                export_options=export_options,
-            )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
-
-    def test_fake_tensor_mode_huggingface_bigscience_bloom(self):
-        config = transformers.BloomConfig()
-        batch, seq = 4, 256
-
-        with torch.onnx.enable_fake_mode() as fake_context:
-            model = transformers.BloomModel(config).eval()
-            input_ids = torch.randint(0, config.vocab_size, (batch, seq))
-            attention_mask = torch.ones(batch, seq, dtype=torch.bool)
-
-            export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
-                model,
-                input_ids=input_ids,
-                attention_mask=attention_mask,
-                export_options=export_options,
-            )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
-
-    def test_fake_tensor_mode_huggingface_open_llama(self):
-        config = transformers.OpenLlamaConfig(
+    @pytorch_test_common.xfail(
+        error_message="Dynamic control flow is not supported at the moment."
+    )
+    def test_fake_tensor_mode_huggingface_llama(self):
+        config = transformers.LlamaConfig(
             vocab_size=8096, hidden_size=256, num_hidden_layers=2, num_attention_heads=2
         )
         batch, seq = 4, 256
 
         with torch.onnx.enable_fake_mode() as fake_context:
-            model = transformers.OpenLlamaModel(config).eval()
+            model = transformers.LlamaModel(config).eval()
             input_ids = torch.randint(0, config.vocab_size, (batch, seq))
             attention_mask = torch.ones(batch, seq, dtype=torch.bool)
             position_ids = torch.arange(0, seq, dtype=torch.long)
             position_ids = position_ids.unsqueeze(0).view(-1, seq)
 
             export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
+            onnx_program = torch.onnx.dynamo_export(
                 model,
                 input_ids=input_ids,
                 attention_mask=attention_mask,
                 position_ids=position_ids,
                 export_options=export_options,
             )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
+            onnx.checker.check_model(onnx_program.model_proto)
+            onnx.shape_inference.infer_shapes(onnx_program.model_proto)
 
-    def test_fake_tensor_mode_huggingface_google_t5(self):
-        config = transformers.T5Config(
-            vocab_size=8096, d_model=256, num_layers=2, num_heads=2
-        )
-        device = "cpu"
-        batch, seq = 4, 256
-        with torch.onnx.enable_fake_mode() as fake_context:
-            model = transformers.T5Model(config).to(device).eval()
-            input_ids = torch.randint(0, config.vocab_size, (batch, seq))
-            attention_mask = torch.ones((batch, seq), dtype=torch.bool)
-            decoder_input_ids = torch.randint(0, config.vocab_size, (batch, seq))
-            export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
-                model,
-                input_ids=input_ids,
-                attention_mask=attention_mask,
-                decoder_input_ids=decoder_input_ids,
-                export_options=export_options,
-            )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
-
-    def test_fake_tensor_mode_huggingface_openai_whisper(self):
-        config = transformers.WhisperConfig()
-        feature_extractor = transformers.WhisperFeatureExtractor()
-        device = "cpu"
-        batch = 4
-        with torch.onnx.enable_fake_mode() as fake_context:
-            input_features = torch.randn(
-                (
-                    batch,
-                    feature_extractor.feature_size,
-                    feature_extractor.nb_max_frames,
-                ),
-                dtype=torch.float32,
-            )
-            decoder_input_ids = torch.tensor([[1, 1]]) * config.decoder_start_token_id
-            model = transformers.AutoModel.from_config(config).to(device).eval()
-            export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
-                model,
-                input_features,
-                decoder_input_ids=decoder_input_ids,
-                export_options=export_options,
-            )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
-
-    # TODO: From Config/Model
-    @pytorch_test_common.skip_in_ci(
-        "Not decorated with xfail because CI doesn't have enough memory to run and then fail."
-        "SymFloat in OnnxFUnction attribute is not supported yet."
-    )
-    def test_fake_tensor_mode_huggingface_databricks_dolly_v2_3b(self):
-        # TODO: Make this test work with config
-        # Dolly has no config on transformers
-        model_name = "databricks/dolly-v2-3b"
-        device = "cpu"
-        with torch.onnx.enable_fake_mode() as fake_context:
-            tokenizer = transformers.AutoTokenizer.from_pretrained(model_name)
-            inputs = tokenizer("Hello world!", return_tensors="pt")
-            model = transformers.AutoModel.from_pretrained(model_name).to(device).eval()
-
-            export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
-                model, **inputs, export_options=export_options
-            )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
-
-    @pytorch_test_common.skip_in_ci(
-        "Not decorated with xfail because CI doesn't have enough memory to run and then fail."
-        "AssertionError: Mutating module attribute seq_len_cached during export."
-        "self.seq_len_cached = seq_len"
+    @pytorch_test_common.xfail(
+        error_message="Dynamic control flow is not supported at the moment."
     )
     def test_fake_tensor_mode_huggingface_tiiuae_falcon(self):
         config = transformers.FalconConfig()
@@ -562,44 +521,238 @@ class TestFxToOnnx(pytorch_test_common.ExportTestCase):
             attention_mask = torch.ones(batch, seq, dtype=torch.bool)
 
             export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
+            onnx_program = torch.onnx.dynamo_export(
                 model,
                 input_ids=input_ids,
                 attention_mask=attention_mask,
                 export_options=export_options,
             )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
+            onnx.checker.check_model(onnx_program.model_proto)
+            onnx.shape_inference.infer_shapes(onnx_program.model_proto)
 
-    @pytorch_test_common.skip_in_ci(
-        "Not decorated with xfail because CI doesn't have enough memory to run and then fail."
-        "torch._dynamo.exc.UserError: Dynamic control flow is not supported at the moment. "
-        "Please use functorch.experimental.control_flow.cond to explicitly capture the control flow"
+    def test_exported_program_torch_distributions_normal_Normal(self):
+        class Model(torch.nn.Module):
+            def __init__(self) -> None:
+                self.normal = torch.distributions.normal.Normal(0, 1)
+                super().__init__()
+
+            def forward(self, x):
+                return self.normal.sample(x.shape)
+
+        x = torch.randn(2, 3)
+        with torch.no_grad():
+            exported_program = torch.export.export(Model(), args=(x,))
+            _ = torch.onnx.dynamo_export(
+                exported_program,
+                x,
+            )
+
+    def test_aten_div_no_opmath_type_promotion(self):
+        class Model(torch.nn.Module):
+            def forward(self, input):
+                return input / 2
+
+        model = Model()
+        input = torch.randn(3, 5, requires_grad=True, dtype=torch.float16)
+
+        model_proto = torch.onnx.dynamo_export(model, input).model_proto
+        model_proto = onnx.inliner.inline_local_functions(model_proto)
+        div_node = next(
+            node for node in model_proto.graph.node if node.op_type == "Div"
+        )
+        # The input of Div node should be the input of the model,
+        # with no Cast node in between.
+        self.assertEqual(div_node.input[0], model_proto.graph.input[0].name)
+
+    @common_utils.parametrize(
+        "float8_type",
+        [
+            common_utils.subtest(
+                torch.float8_e5m2,
+                name="torch_float8_e5m2",
+            ),
+            common_utils.subtest(
+                torch.float8_e5m2fnuz,
+                name="torch_float8_e5m2fnuz",
+            ),
+            common_utils.subtest(
+                torch.float8_e4m3fn,
+                name="torch_float8_e4m3fn",
+            ),
+            common_utils.subtest(
+                torch.float8_e4m3fnuz,
+                name="torch_float8_e4m3fnuz",
+            ),
+        ],
     )
-    def test_fake_tensor_mode_huggingface_mosaicml_mpt_7b(self):
-        # TODO: Make this test work with config
-        # mpt-7b has no config on transformers
-        model_name = "mosaicml/mpt-7b"
-        device = "cpu"
-        with torch.onnx.enable_fake_mode() as fake_context:
-            tokenizer = transformers.AutoTokenizer.from_pretrained(
-                model_name, trust_remote_code=True
+    def test_float8_support(self, float8_type):
+        class Float8Module(torch.nn.Module):
+            def forward(self, input: torch.Tensor):
+                input = input.to(float8_type)
+                return input + torch.tensor(1.0, dtype=float8_type)
+
+        # NOTE: shape inference error raised in optimizer due to unsupported dtype
+        with self.assertWarnsOnceRegex(
+            UserWarning, "ONNXScript optimizer failed. Skipping optimization."
+        ):
+            _ = torch.onnx.dynamo_export(Float8Module(), torch.randn(1, 2, 3, 4))
+
+    def test_export_with_logging_logger(self):
+        logger = logging.getLogger(__name__)
+
+        class LoggingLoggerModule(torch.nn.Module):
+            def forward(self, x):
+                logger.log("abc")
+                return x + 1
+
+        input = torch.randn(2, 3)
+        model = LoggingLoggerModule()
+        _ = torch.onnx.dynamo_export(model, input)
+
+    def test_export_with_hf_logging_logger(self):
+        logger = transformers.utils.logging.get_logger(__name__)
+
+        class HFLoggingLoggerModule(torch.nn.Module):
+            def forward(self, x):
+                logger.warning_once("abc")
+                return x + 1
+
+        input = torch.randn(2, 3)
+        model = HFLoggingLoggerModule()
+        _ = torch.onnx.dynamo_export(model, input)
+
+    def test_checkpoint_cast(self):
+        model_id = "openai/whisper-large-v3"
+        feature_extractor = transformers.WhisperFeatureExtractor(feature_size=128)
+        batch = 4
+
+        with torch.onnx.enable_fake_mode() as ctx:
+            model = transformers.AutoModelForSpeechSeq2Seq.from_pretrained(
+                model_id, low_cpu_mem_usage=False, use_safetensors=False
             )
-            inputs = tokenizer("Hello world!", return_tensors="pt")
-            model = (
-                transformers.AutoModelForCausalLM.from_pretrained(
-                    model_name, trust_remote_code=True
-                )
-                .to(device)
-                .eval()
+            input = {
+                "input_features": torch.randn(
+                    (
+                        batch,
+                        feature_extractor.feature_size,
+                        feature_extractor.nb_max_frames,
+                    )
+                ),
+                "decoder_input_ids": torch.tensor([[1, 1]]) * 8001,
+                "return_dict": False,
+            }
+
+        export_options = torch.onnx.ExportOptions(fake_context=ctx)
+        onnx_program = torch.onnx.dynamo_export(
+            model, **input, export_options=export_options
+        )
+        with tempfile.NamedTemporaryFile(suffix=".onnx") as tmp_onnx_file:
+            onnx_program.save(tmp_onnx_file.name)
+            onnx.checker.check_model(tmp_onnx_file.name, full_check=True)
+
+    @common_utils.parametrize(
+        "include_initializer",
+        [
+            common_utils.subtest(
+                True,
+                name="include_initializer",
+            ),
+            common_utils.subtest(
+                False,
+                name="dont_include_initializer",
+            ),
+        ],
+    )
+    @common_utils.parametrize(
+        "use_fake_mode",
+        [
+            common_utils.subtest(
+                True,
+                name="use_fake_mode",
+            ),
+            common_utils.subtest(
+                False,
+                name="no_fake_mode",
+            ),
+        ],
+    )
+    @common_utils.parametrize(
+        "use_exported_program",
+        [
+            common_utils.subtest(
+                True,
+                name="use_exported_program",
+            ),
+            common_utils.subtest(
+                False,
+                name="no_exported_program",
+            ),
+        ],
+    )
+    def test_save_with_without_initializer(
+        self, include_initializer, use_fake_mode, use_exported_program
+    ):
+        class MNISTModel(nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+                self.conv1 = nn.Conv2d(1, 32, 3, 1, bias=False)
+                self.conv2 = nn.Conv2d(32, 64, 3, 1, bias=False)
+                self.fc1 = nn.Linear(9216, 128, bias=False)
+                self.fc2 = nn.Linear(128, 10, bias=False)
+
+            def forward(self, tensor_x: torch.Tensor):
+                tensor_x = self.conv1(tensor_x)
+                tensor_x = F.sigmoid(tensor_x)
+                tensor_x = self.conv2(tensor_x)
+                tensor_x = F.sigmoid(tensor_x)
+                tensor_x = F.max_pool2d(tensor_x, 2)
+                tensor_x = torch.flatten(tensor_x, 1)
+                tensor_x = self.fc1(tensor_x)
+                tensor_x = F.sigmoid(tensor_x)
+                tensor_x = self.fc2(tensor_x)
+                output = F.log_softmax(tensor_x, dim=1)
+                return output
+
+        state_dict = MNISTModel().state_dict()
+        if use_fake_mode:
+            with torch.onnx.enable_fake_mode() as ctx:
+                model = MNISTModel()
+                tensor_x = torch.rand((64, 1, 28, 28), dtype=torch.float32)
+                if use_exported_program:
+                    model = torch.export.export(model, args=(tensor_x,))
+                export_options = torch.onnx.ExportOptions(fake_context=ctx)
+        else:
+            model = MNISTModel()
+            tensor_x = torch.rand((64, 1, 28, 28), dtype=torch.float32)
+            if use_exported_program:
+                model = torch.export.export(model, args=(tensor_x,))
+            export_options = torch.onnx.ExportOptions()
+
+        onnx_program = torch.onnx.dynamo_export(
+            model, tensor_x, export_options=export_options
+        )
+        onnx_program.apply_weights(state_dict)
+        with tempfile.NamedTemporaryFile(suffix=".onnx") as tmp_onnx_file:
+            onnx_program.save(
+                tmp_onnx_file.name,
+                include_initializers=include_initializer,
+            )
+            onnx_model = onnx.load(tmp_onnx_file.name)
+            self.assertEqual(
+                (include_initializer and len(onnx_model.graph.initializer) > 0)
+                or (not include_initializer and len(onnx_model.graph.initializer) == 0),
+                True,
             )
 
-            export_options = torch.onnx.ExportOptions(fake_context=fake_context)
-            export_output = torch.onnx.dynamo_export(
-                model, **inputs, export_options=export_options
-            )
-            onnx.checker.check_model(export_output.model_proto)
-            onnx.shape_inference.infer_shapes(export_output.model_proto)
+    def test_export_with_print(self):
+        class PrintModule(torch.nn.Module):
+            def forward(self, x):
+                print("abc")
+                return x + 1
+
+        input = torch.randn(2, 3)
+        model = PrintModule()
+        _ = torch.onnx.dynamo_export(model, input)
 
 
 if __name__ == "__main__":
