@@ -6,6 +6,8 @@
 
 #include <c10/util/irange.h>
 
+#include <utility>
+
 namespace torch::jit::tensorexpr {
 
 RegisterCodeGen<SimpleIREvaluator> ir_eval_codegen_reg("simple_ir_eval");
@@ -22,15 +24,14 @@ int64_t InterpValue::intValue() const {
 }
 
 template <typename T>
-inline typename std::enable_if<std::is_integral<T>::value, T>::type mod_value(
-    T lhs,
-    T rhs) {
+inline std::enable_if_t<std::is_integral_v<T>, T> mod_value(T lhs, T rhs) {
   return lhs % rhs;
 }
 
 template <typename T>
-inline typename std::enable_if<std::is_floating_point<T>::value, T>::type
-mod_value(T lhs, T rhs) {
+inline std::enable_if_t<std::is_floating_point_v<T>, T> mod_value(
+    T lhs,
+    T rhs) {
   return std::fmod(lhs, rhs);
 }
 
@@ -39,17 +40,14 @@ inline bool mod_value(bool lhs, bool rhs) {
 }
 
 template <typename T>
-inline typename std::enable_if<std::is_integral<T>::value, T>::type div_value(
-    T lhs,
-    T rhs) {
+inline std::enable_if_t<std::is_integral_v<T>, T> div_value(T lhs, T rhs) {
   TORCH_CHECK(rhs != 0, "Division by zero");
   return lhs / rhs;
 }
 
 template <typename T>
-inline typename std::enable_if<std::is_floating_point<T>::value, T>::
-    type __ubsan_ignore_float_divide_by_zero__
-    div_value(T lhs, T rhs) {
+inline std::enable_if_t<std::is_floating_point_v<T>, T>
+    __ubsan_ignore_float_divide_by_zero__ div_value(T lhs, T rhs) {
   return lhs / rhs;
 }
 
@@ -66,25 +64,23 @@ inline c10::BFloat16 div_value(c10::BFloat16 lhs, c10::BFloat16 rhs) {
   return lhs / rhs;
 }
 
-// NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
 class SimpleIREvaluatorImpl : public IRVisitor {
  public:
-  // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
   SimpleIREvaluatorImpl() = default;
 
   ~SimpleIREvaluatorImpl() override = default;
 
-  void bindBuf(BufPtr buf, void* ptr) {
+  void bindBuf(const BufPtr& buf, void* ptr) {
     GRAPH_DEBUG("Binding ptr ", ptr, " with buf ", buf->name_hint());
     buffer_mapping_[buf] = ptr;
   }
-  void bindVar(VarPtr var, const InterpValue& val) {
+  void bindVar(const VarPtr& var, const InterpValue& val) {
     eval_context_[var] = val;
     GRAPH_DEBUG(
         "Binding value ", val.intValue(), " with var ", var->name_hint());
   }
 
-  InterpValue evaluateExpr(ExprPtr e) {
+  InterpValue evaluateExpr(const ExprPtr& e) {
     e->accept(this);
     return value_;
   }
@@ -99,71 +95,71 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     internal_buffers_.clear();
   }
 
-  TORCH_API void visit(AddPtr v) override {
+  TORCH_API void visit(const AddPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(SubPtr v) override {
+  TORCH_API void visit(const SubPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(MulPtr v) override {
+  TORCH_API void visit(const MulPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(DivPtr v) override {
+  TORCH_API void visit(const DivPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(ModPtr v) override {
+  TORCH_API void visit(const ModPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(MaxPtr v) override {
+  TORCH_API void visit(const MaxPtr& v) override {
     visit_binary_op(v, v->propagate_nans());
   }
-  TORCH_API void visit(MinPtr v) override {
+  TORCH_API void visit(const MinPtr& v) override {
     visit_binary_op(v, v->propagate_nans());
   }
 
-  TORCH_API void visit(AndPtr v) override {
+  TORCH_API void visit(const AndPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(OrPtr v) override {
+  TORCH_API void visit(const OrPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(XorPtr v) override {
+  TORCH_API void visit(const XorPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(LshiftPtr v) override {
+  TORCH_API void visit(const LshiftPtr& v) override {
     visit_binary_op(v);
   }
-  TORCH_API void visit(RshiftPtr v) override {
+  TORCH_API void visit(const RshiftPtr& v) override {
     visit_binary_op(v);
   }
 
-  void visit(CompareSelectPtr v) override {
+  void visit(const CompareSelectPtr& v) override {
     visit_compare_select_op(v, v->compare_select_op());
   }
 
   template <typename T>
-  typename std::enable_if_t<std::is_floating_point<T>::value, T> max_value(
+  typename std::enable_if_t<std::is_floating_point_v<T>, T> max_value(
       T a,
       T b) {
     return std::isnan(a) ? a : (std::isnan(b) ? b : (a < b ? b : a));
   }
 
   template <typename T>
-  typename std::enable_if_t<!std::is_floating_point<T>::value, T> max_value(
+  typename std::enable_if_t<!std::is_floating_point_v<T>, T> max_value(
       T a,
       T b) {
     return a < b ? b : a;
   }
 
   template <typename T>
-  typename std::enable_if_t<std::is_floating_point<T>::value, T> min_value(
+  typename std::enable_if_t<std::is_floating_point_v<T>, T> min_value(
       T a,
       T b) {
     return std::isnan(a) ? a : (std::isnan(b) ? b : (a < b ? a : b));
   }
 
   template <typename T>
-  typename std::enable_if_t<!std::is_floating_point<T>::value, T> min_value(
+  typename std::enable_if_t<!std::is_floating_point_v<T>, T> min_value(
       T a,
       T b) {
     return a < b ? a : b;
@@ -246,8 +242,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     for (const auto i : c10::irange(lhs_v.size())) {
       switch (op_type) {
         case IRNodeType::kLshift: {
-          typename std::make_unsigned<T>::type a =
-              static_cast<typename std::make_unsigned<T>::type>(lhs_v[i]);
+          auto a = static_cast<std::make_unsigned_t<T>>(lhs_v[i]);
           result_v[i] = a << rhs_v[i];
           break;
         }
@@ -304,9 +299,9 @@ class SimpleIREvaluatorImpl : public IRVisitor {
 
   template <
       typename D,
-      typename std::enable_if<std::is_same<
+      std::enable_if_t<std::is_same_v<
           decltype(detail::bin_op_deducer(std::declval<D>())),
-          void>::value>::type* = nullptr>
+          void>>* = nullptr>
   void visit_binary_op(NodePtr<D> v, bool option = false) {
     v->lhs()->accept(this);
     InterpValue lhs_v = value_;
@@ -390,7 +385,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
   }
 
   void visit_compare_select_op(
-      CompareSelectPtr v,
+      const CompareSelectPtr& v,
       CompareSelectOperation cmp_op) {
     v->lhs()->accept(this);
     InterpValue lhs_v = value_;
@@ -419,14 +414,14 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-#define IMM_VISIT(Type, Name)                     \
-  TORCH_API void visit(Name##ImmPtr v) override { \
-    value_ = InterpValue(v->value());             \
+#define IMM_VISIT(Type, Name)                            \
+  TORCH_API void visit(const Name##ImmPtr& v) override { \
+    value_ = InterpValue(v->value());                    \
   }
   AT_FORALL_SCALAR_TYPES_AND3(Bool, Half, BFloat16, IMM_VISIT);
 #undef IMM_VISIT
 
-  TORCH_API void visit(BlockPtr v) override {
+  TORCH_API void visit(const BlockPtr& v) override {
     BlockPtr last = scope_;
     scope_ = v;
     for (const StmtPtr& s : v->stmts()) {
@@ -444,7 +439,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     scope_ = last;
   }
 
-  TORCH_API void visit(VarPtr v) override {
+  TORCH_API void visit(const VarPtr& v) override {
     auto iter = eval_context_.find(v);
     if (iter == eval_context_.end()) {
       throw malformed_input("could not find Var in context", v);
@@ -497,7 +492,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  TORCH_API void visit(CastPtr v) override {
+  TORCH_API void visit(const CastPtr& v) override {
     ExprPtr src_value = v->src_value();
     src_value->accept(this);
     Dtype dst_dtype = v->dtype();
@@ -552,7 +547,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  TORCH_API void visit(BitCastPtr v) override {
+  TORCH_API void visit(const BitCastPtr& v) override {
     ExprPtr src_value = v->src_value();
     src_value->accept(this);
     Dtype dst_dtype = v->dtype();
@@ -575,7 +570,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  TORCH_API void visit(ForPtr v) override {
+  TORCH_API void visit(const ForPtr& v) override {
     ExprPtr var_node = v->var();
     v->start()->accept(this);
     auto dtype = value_.dtype();
@@ -595,14 +590,14 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     eval_context_.erase(var_node);
   }
 
-  TORCH_API void visit(RampPtr v) override {
+  TORCH_API void visit(const RampPtr& v) override {
     v->base()->accept(this);
     auto base = value().intValue();
     v->stride()->accept(this);
     auto stride = value().intValue();
     int lanes = v->lanes();
 
-    std::vector<int> values(lanes);
+    std::vector<int64_t> values(lanes);
     for (const auto i : c10::irange(lanes)) {
       values[i] = base + i * stride;
     }
@@ -610,7 +605,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     value_ = InterpValue(values);
   }
 
-  TORCH_API void visit(BroadcastPtr v) override {
+  TORCH_API void visit(const BroadcastPtr& v) override {
     v->value()->accept(this);
     InterpValue value = this->value();
     int lanes = v->lanes();
@@ -627,10 +622,9 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  TORCH_API void visit(IfThenElsePtr v) override {
+  TORCH_API void visit(const IfThenElsePtr& v) override {
     v->condition()->accept(this);
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    bool cond_v;
+    bool cond_v = false;
     switch (value_.dtype().scalar_type()) {
 #define TYPE_CASE(Type, Name)   \
   case ScalarType::Name: {      \
@@ -688,7 +682,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
           throw malformed_input(
               "Number of dimensions did not match number of strides", buf);
         }
-        size_t buf_size = 1;
+        int64_t buf_size = 1;
         if (!dims.empty()) {
           ExprHandle buf_size_expr = ExprHandle(immLike(dims[0], 1));
           ExprHandle negative_one = ExprHandle(immLike(dims[0], -1));
@@ -731,7 +725,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  TORCH_API void visit(LoadPtr v) override {
+  TORCH_API void visit(const LoadPtr& v) override {
     auto iter = buffer_mapping_.find(v->buf());
     if (iter == buffer_mapping_.end()) {
       throw malformed_input("could not find base node in Load", v);
@@ -765,9 +759,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     value_ = InterpValue(val);                       \
   } break;
       AT_FORALL_SCALAR_TYPES_AND3(Bool, Half, BFloat16, TYPE_CASE);
-      // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
       TYPE_CASE(c10::quint8, QUInt8);
-      // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
       TYPE_CASE(c10::qint8, QInt8);
 #undef TYPE_CASE
       default:
@@ -775,7 +767,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  TORCH_API void visit(StorePtr v) override {
+  TORCH_API void visit(const StorePtr& v) override {
     auto iter = buffer_mapping_.find(v->buf());
     if (iter == buffer_mapping_.end()) {
       throw malformed_input("could not find base node in Store", v);
@@ -814,9 +806,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }                                                           \
   } break;
       AT_FORALL_SCALAR_TYPES_AND3(Bool, Half, BFloat16, TYPE_CASE);
-      // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
       TYPE_CASE(c10::quint8, QUInt8);
-      // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
       TYPE_CASE(c10::qint8, QInt8);
 #undef TYPE_CASE
       default:
@@ -824,7 +814,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  void visit(ExternalCallPtr v) override {
+  void visit(const ExternalCallPtr& v) override {
     auto& func_registry = getNNCFunctionRegistry();
     if (!func_registry.count(v->func_name())) {
       throw unimplemented_lowering(v);
@@ -865,8 +855,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
     for (const ExprPtr& a : v->args()) {
       a->accept(this);
-      // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-      int64_t val;
+      int64_t val = 0;
       if (value().dtype() == kLong) {
         val = value().as<int64_t>();
       } else if (value().dtype() == kInt) {
@@ -896,7 +885,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
         extra_args.data());
   }
 
-  void visit(ExternalCallWithAllocPtr v) override {
+  void visit(const ExternalCallWithAllocPtr& v) override {
     auto& func_registry = getNNCFunctionRegistry();
     if (!func_registry.count(v->func_name())) {
       throw unimplemented_lowering(v);
@@ -937,8 +926,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
     for (const auto& a : v->args()) {
       a->accept(this);
-      // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-      int64_t val;
+      int64_t val = 0;
       if (value().dtype() == kLong) {
         val = value().as<int64_t>();
       } else if (value().dtype() == kInt) {
@@ -958,14 +946,12 @@ class SimpleIREvaluatorImpl : public IRVisitor {
 
     auto fn_ptr = func_registry.at(v->func_name());
     (*fn_ptr)(
-        // @lint-ignore CLANGTIDY
         bufs_in_size,
         buf_ptrs.data(),
         buf_ranks.data(),
         buf_dims.data(),
         buf_strides.data(),
         buf_dtypes.data(),
-        // @lint-ignore CLANGTIDY
         extra_args.size(),
         extra_args.data());
 
@@ -977,7 +963,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
   }
 
   template <typename TReturn, typename TInput>
-  void visit_intrinsics_helper(IntrinsicsPtr v) {
+  void visit_intrinsics_helper(const IntrinsicsPtr& v) {
     std::vector<InterpValue> values(v->nparams());
     for (const auto i : c10::irange(v->nparams())) {
       v->param(i)->accept(this);
@@ -1012,7 +998,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     value_ = InterpValue(result);
   }
 
-  TORCH_API void visit(IntrinsicsPtr v) override {
+  TORCH_API void visit(const IntrinsicsPtr& v) override {
     auto ty = v->dtype().scalar_type();
     if (v->op_type() == kIsNan) {
       auto inp_dtype = v->params().at(0)->dtype().scalar_type();
@@ -1039,7 +1025,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     }
   }
 
-  void visit(AllocatePtr v) override {
+  void visit(const AllocatePtr& v) override {
     BufPtr b = v->buf();
     std::vector<ExprPtr> dims = b->dims();
     int64_t total_byte_size = b->dtype().byte_size();
@@ -1050,7 +1036,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     auto int_count = (total_byte_size + sizeof(int) - 1) / sizeof(int);
     GRAPH_DEBUG(
         "ALLOCATE: buf=", v->buf()->name_hint(), ", size=", total_byte_size);
-    std::unique_ptr<std::vector<int>> buffer(new std::vector<int>(int_count));
+    auto buffer = std::make_unique<std::vector<int>>(int_count);
     auto iter = buffer_mapping_.find(b);
     if (iter != buffer_mapping_.end() && iter->second != nullptr) {
       throw std::runtime_error(
@@ -1061,14 +1047,14 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     internal_buffers_.insert(std::make_pair(b, std::move(buffer)));
   }
 
-  void visit(PlacementAllocatePtr v) override {
+  void visit(const PlacementAllocatePtr& v) override {
     buffer_mapping_[v->buf()] = buffer_mapping_.at(v->buf_to_reuse());
   }
 
-  void visit(FreePtr v) override {
+  void visit(const FreePtr& v) override {
     BufPtr b = v->buf();
     GRAPH_DEBUG("FREE: buf=", v->buf()->name_hint());
-    int count = internal_buffers_.erase(b);
+    auto count = internal_buffers_.erase(b);
     if (count == 0) {
       throw std::runtime_error(
           "Free a buffer that is not currently bound: " +
@@ -1077,7 +1063,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     buffer_mapping_.erase(b);
   }
 
-  void visit(FreeExtPtr v) override {
+  void visit(const FreeExtPtr& v) override {
     const auto& bufs = v->bufs();
     const auto bufs_num = bufs.size();
     std::vector<void*> buf_ptrs;
@@ -1092,12 +1078,12 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     nnc_aten_free(bufs_num, buf_ptrs.data());
   }
 
-  void visit(LetPtr v) override {
+  void visit(const LetPtr& v) override {
     var_by_scope_[scope_].push_back(v->var());
     bindVar(v->var(), evaluateExpr(v->value()));
   }
 
-  void visit(CondPtr v) override {
+  void visit(const CondPtr& v) override {
     v->condition()->accept(this);
     if (value().intValue()) {
       if (v->true_stmt()) {
@@ -1114,8 +1100,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
   template <
       typename TReturn,
       typename TInput,
-      typename std::enable_if<std::is_floating_point<TInput>::value, int>::
-          type = 0>
+      std::enable_if_t<std::is_floating_point_v<TInput>, int> = 0>
   static TReturn compute_intrinsics(IntrinsicsOp op_type, TInput v) {
     switch (op_type) {
       case kSin:
@@ -1178,27 +1163,25 @@ class SimpleIREvaluatorImpl : public IRVisitor {
       case kIsNan:
         return std::isnan(v);
       default:
-        throw std::runtime_error("Invalid op_type: " + c10::to_string(op_type));
+        throw std::runtime_error("Invalid op_type: " + std::to_string(op_type));
     }
   }
 
   template <
       typename TReturn,
       typename TInput,
-      typename std::enable_if<std::is_integral<TInput>::value, int>::type = 0>
+      std::enable_if_t<std::is_integral_v<TInput>, int> = 0>
   static TReturn compute_intrinsics(IntrinsicsOp op_type, TInput v) {
     switch (op_type) {
       case kAbs: {
         // internal tool complains about calling `abs` on unsigned, the
         // following makes the tool happy
-        using X =
-            std::conditional_t<std::is_unsigned<TInput>::value, int, TInput>;
-        return std::is_unsigned<TInput>::value ? v
-                                               : std::abs(static_cast<X>(v));
+        using X = std::conditional_t<std::is_unsigned_v<TInput>, int, TInput>;
+        return std::is_unsigned_v<TInput> ? v : std::abs(static_cast<X>(v));
       }
       default:
         throw std::runtime_error(
-            "Invalid integral op_type: " + c10::to_string(op_type));
+            "Invalid integral op_type: " + std::to_string(op_type));
     }
   }
 
@@ -1208,7 +1191,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
       case kIsNan:
         return std::isnan(v);
       default:
-        throw std::runtime_error("Invalid op_type: " + c10::to_string(op_type));
+        throw std::runtime_error("Invalid op_type: " + std::to_string(op_type));
     }
   }
 
@@ -1224,7 +1207,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
       case kAtan2:
         return std::atan2(v1, v2);
       default:
-        throw std::runtime_error("Invalid op_type: " + c10::to_string(op_type));
+        throw std::runtime_error("Invalid op_type: " + std::to_string(op_type));
     }
   }
 
@@ -1243,7 +1226,7 @@ SimpleIREvaluator::SimpleIREvaluator(
     const std::vector<BufferArg>& buffer_args,
     at::Device device,
     const std::string& kernel_func_name)
-    : CodeGen(stmt, buffer_args, device, kernel_func_name) {
+    : CodeGen(std::move(stmt), buffer_args, device, kernel_func_name) {
   impl_ = std::make_unique<SimpleIREvaluatorImpl>();
   expand_intrinsics();
 }
@@ -1292,7 +1275,7 @@ void SimpleIREvaluator::bindArg(const BufferArg& bufArg, void* data) {
   }
 }
 
-void SimpleIREvaluator::bindVar(VarPtr v, ExprPtr e) {
+void SimpleIREvaluator::bindVar(const VarPtr& v, const ExprPtr& e) {
   impl_->bindVar(v, impl_->evaluateExpr(e));
 }
 
@@ -1300,12 +1283,12 @@ InterpValue SimpleIREvaluator::value() const {
   return impl_->value();
 }
 
-c10::optional<int64_t> evalInt(ExprPtr e) {
+std::optional<int64_t> evalInt(ExprPtr e) {
   try {
-    return ExprEval<SimpleIREvaluator>(cast<int64_t>(ExprHandle(e)))
+    return ExprEval<SimpleIREvaluator>(cast<int64_t>(ExprHandle(std::move(e))))
         .value<int64_t>();
   } catch (std::runtime_error& err) {
-    return c10::nullopt;
+    return std::nullopt;
   }
 }
 
