@@ -6,15 +6,13 @@
 
 using namespace torch::jit::tensorexpr;
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
+namespace torch::jit::tensorexpr {
 namespace {
 std::vector<int64_t> _pair_int(ArgValue v) {
-  if (auto t = c10::get_if<IntList>(&v)) {
+  if (auto t = std::get_if<IntList>(&v)) {
     return {(*t)[0], (*t)[1]};
   }
-  auto i = c10::get<int64_t>(v);
+  auto i = std::get<int64_t>(v);
   return {i, i};
 }
 } // namespace
@@ -43,8 +41,8 @@ static BufHandle makeQBufHandleChannelsLast(
     const std::string& name,
     const std::vector<ExprHandle>& dims,
     Dtype dtype,
-    const ExprPtr qscale,
-    const ExprPtr qzero) {
+    const ExprPtr& qscale,
+    const ExprPtr& qzero) {
   BufHandle ResultBuf(name, dims, dtype);
   ResultBuf.node()->set_qscale(qscale);
   ResultBuf.node()->set_qzero(qzero);
@@ -70,8 +68,8 @@ static BufHandle makeQBufHandleContiguous(
     const std::string& name,
     const std::vector<ExprHandle>& dims,
     Dtype dtype,
-    const ExprPtr qscale,
-    const ExprPtr qzero) {
+    const ExprPtr& qscale,
+    const ExprPtr& qzero) {
   BufHandle ResultBuf(name, dims, dtype);
   ResultBuf.node()->set_qscale(qscale);
   ResultBuf.node()->set_qzero(qzero);
@@ -109,7 +107,7 @@ static bool isChannelsLast(const BufHandle& buf) {
 }
 
 static ExprHandle quant(
-    ExprHandle x,
+    const ExprHandle& x,
     Dtype out_dtype,
     ExprHandle qscale,
     ExprHandle qzero) {
@@ -141,7 +139,7 @@ Tensor computeQuantizePerTensor(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>&,
+    const std::optional<ScalarType>&,
     at::Device) {
   std::vector<VarPtr> vars;
   std::vector<ExprHandle> indices;
@@ -161,7 +159,7 @@ Tensor computeQuantizePerTensor(
       return Dtype(ScalarType::QUInt8);
     }
     throw malformed_input("Expected quantized dtype");
-  }(c10::get<int64_t>(inputs[3]));
+  }(std::get<int64_t>(inputs[3]));
 
   ExprHandle e =
       quant(tensorOrConstant(inputs[0], indices), dtype, qscale, qzero);
@@ -171,7 +169,7 @@ Tensor computeQuantizePerTensor(
       ExprHandleVectorToExprVector(outputShape),
       dtype,
       nullptr,
-      c10::nullopt,
+      std::nullopt,
       qscale.node(),
       qzero.node());
   return Tensor(buf, vars, e.node());
@@ -181,16 +179,16 @@ Tensor computeQuantizedAdd(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
-  const BufHandle& QA = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& QB = c10::get<BufHandle>(inputs[1]);
+  const BufHandle& QA = std::get<BufHandle>(inputs[0]);
+  const BufHandle& QB = std::get<BufHandle>(inputs[1]);
   auto qa_scale = ExprHandle(QA.node()->qscale());
   auto qa_zero = ExprHandle(QA.node()->qzero());
   auto qb_scale = ExprHandle(QB.node()->qscale());
   auto qb_zero = ExprHandle(QB.node()->qzero());
-  ExprHandle out_qscale = DoubleImm::make(c10::get<double>(inputs[2]));
-  ExprHandle out_qzero = LongImm::make(c10::get<int64_t>(inputs[3]));
+  ExprHandle out_qscale = DoubleImm::make(std::get<double>(inputs[2]));
+  ExprHandle out_qzero = LongImm::make(std::get<int64_t>(inputs[3]));
   Dtype dequant_dtype = kFloat;
   Dtype out_dtype = outputType ? Dtype(*outputType) : QA.dtype();
   std::vector<VarPtr> vars;
@@ -224,13 +222,12 @@ Tensor computeQuantizePerTensorExternalCall(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
-  const BufHandle& x = c10::get<BufHandle>(inputs[0]);
-  const auto qscale = c10::get<double>(inputs[1]);
-  const auto qzero = c10::get<int64_t>(inputs[2]);
-  const auto qdtype = c10::get<int64_t>(inputs[3]);
+  const BufHandle& x = std::get<BufHandle>(inputs[0]);
+  const auto qscale = std::get<double>(inputs[1]);
+  const auto qzero = std::get<int64_t>(inputs[2]);
+  const auto qdtype = std::get<int64_t>(inputs[3]);
 
   const auto dtype = [](auto qdtype) {
     if (static_cast<int64_t>(ScalarType::QInt8) == qdtype) {
@@ -257,14 +254,14 @@ Tensor computeDequantizeExternalCall(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
   Dtype dtype = kFloat;
   if (outputType) {
     dtype = Dtype(*outputType);
   }
 
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
   const int64_t qdtype = (int64_t)immQDType(qx);
 
   BufHandle ResultBuf("dequantize", outputShape, dtype);
@@ -282,7 +279,7 @@ Tensor computeQuantizedConv2dPrepack(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
   Dtype dtype = kFloat;
   if (outputType) {
@@ -290,12 +287,12 @@ Tensor computeQuantizedConv2dPrepack(
   }
 
   BufHandle ResultBuf("quantized_conv2d_prepack", outputShape, dtype);
-  const BufHandle& qw = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& b = c10::get<BufHandle>(inputs[1]);
+  const BufHandle& qw = std::get<BufHandle>(inputs[0]);
+  const BufHandle& b = std::get<BufHandle>(inputs[1]);
   auto strides = _pair_int(inputs[2]);
   auto padding = _pair_int(inputs[3]);
   auto dilation = _pair_int(inputs[4]);
-  int groups = c10::get<int64_t>(inputs[5]);
+  auto groups = std::get<int64_t>(inputs[5]);
   TORCH_INTERNAL_ASSERT(
       qw.node()->qscale(),
       buildErrorMessage(
@@ -308,17 +305,11 @@ Tensor computeQuantizedConv2dPrepack(
       ResultBuf,
       "nnc_aten_quantized_conv2d_prepack",
       {qw, b},
-      // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
       {strides[0],
-       // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
        strides[1],
-       // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
        padding[0],
-       // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
        padding[1],
-       // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
        dilation[0],
-       // NOLINTNEXTLINE(facebook-hte-LocalUncheckedArrayBounds)
        dilation[1],
        groups,
        immQScale(qw),
@@ -331,14 +322,12 @@ Tensor computeQuantizedConv1d(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& prepacked = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
+  const BufHandle& prepacked = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qx);
   auto ResultBuf = makeQBufHandleChannelsLast(
@@ -363,14 +352,12 @@ Tensor computeQuantizedConv2d(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& prepacked = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
+  const BufHandle& prepacked = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qx);
   auto ResultBuf = makeQBufHandleChannelsLast(
@@ -395,14 +382,12 @@ Tensor computeQuantizedConv2dRelu(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& prepacked = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
+  const BufHandle& prepacked = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qx);
   auto ResultBuf = makeQBufHandleChannelsLast(
@@ -427,14 +412,12 @@ Tensor computeQuantizedLinear(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& prepacked = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
+  const BufHandle& prepacked = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qx);
   auto ResultBuf = makeQBufHandleContiguous(
@@ -459,14 +442,12 @@ Tensor computeQuantizedLinearRelu(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& prepacked = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
+  const BufHandle& prepacked = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qx);
   auto ResultBuf = makeQBufHandleContiguous(
@@ -491,14 +472,12 @@ Tensor computeQuantizedAddExternalCall(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qa = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& qb = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qa = std::get<BufHandle>(inputs[0]);
+  const BufHandle& qb = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qa);
   const bool isQAChannelsLast = isChannelsLast(qa);
@@ -535,14 +514,12 @@ Tensor computeQuantizedMul(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qa = c10::get<BufHandle>(inputs[0]);
-  const BufHandle& qb = c10::get<BufHandle>(inputs[1]);
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const BufHandle& qa = std::get<BufHandle>(inputs[0]);
+  const BufHandle& qb = std::get<BufHandle>(inputs[1]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qa);
   auto ResultBuf = makeQBufHandleContiguous(
@@ -567,11 +544,10 @@ Tensor computeQuantizedMulScalar(
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
     // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qa = c10::get<BufHandle>(inputs[0]);
-  const auto scalar = c10::get<double>(inputs[1]);
+  const BufHandle& qa = std::get<BufHandle>(inputs[0]);
+  const auto scalar = std::get<double>(inputs[1]);
   // Change to dtype based on outputType when dtype propagation implemented
   const auto out_qdtype = immQDType(qa);
   double scale1 = immQScale(qa);
@@ -593,11 +569,9 @@ Tensor computeQuantizedRelu(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
-    // NOLINTNEXTLINE
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  const BufHandle& qa = c10::get<BufHandle>(inputs[0]);
+  const BufHandle& qa = std::get<BufHandle>(inputs[0]);
   const auto out_qdtype = immQDType(qa);
   const bool isQAChannelsLast = isChannelsLast(qa);
   auto ResultBuf = isQAChannelsLast ? makeQBufHandleChannelsLast(
@@ -625,16 +599,16 @@ Tensor computeQuantizedCat(
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
     // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     // NOLINTNEXTLINE
     at::Device device) {
   // NOLINTNEXTLINE(performance-unnecessary-copy-initialization)
-  auto inputList = c10::get<BufList>(inputs[0]);
-  auto argDim = c10::get<int64_t>(inputs[1]);
+  auto inputList = std::get<BufList>(inputs[0]);
+  auto argDim = std::get<int64_t>(inputs[1]);
   auto n = inputList.size();
   // TODO: handle optional out_qscale, out_qzero
-  const auto out_qscale = c10::get<double>(inputs[2]);
-  const auto out_qzero = c10::get<int64_t>(inputs[3]);
+  const auto out_qscale = std::get<double>(inputs[2]);
+  const auto out_qzero = std::get<int64_t>(inputs[3]);
 
   std::vector<BufHandle> args;
   std::vector<ExprHandle> extra_args;
@@ -663,13 +637,13 @@ Tensor computeDequantize(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
   Dtype dtype = kFloat;
   if (outputType) {
     dtype = Dtype(*outputType);
   }
-  auto qx = c10::get<BufHandle>(inputs[0]);
+  auto qx = std::get<BufHandle>(inputs[0]);
   TORCH_INTERNAL_ASSERT(
       qx.node()->qscale(),
       buildErrorMessage("Missing quantized scale for dequantize"));
@@ -695,9 +669,9 @@ Tensor computeUpsampleNearest2d(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
-  auto A = c10::get<BufHandle>(inputs[0]);
+  auto A = std::get<BufHandle>(inputs[0]);
   const auto& output_height = outputShape[2];
   const auto& output_width = outputShape[3];
   auto input_height = ExprHandle(A.dim(2));
@@ -710,7 +684,7 @@ Tensor computeUpsampleNearest2d(
       promoteToDtype(input_height, ScalarType::Double) / output_height;
   auto scale_w = promoteToDtype(input_width, ScalarType::Double) / output_width;
   // TODO: will repetitive if in idx calculation will be taken out of the loop?
-  auto compute_nearest_idx = [](ExprHandle scale,
+  auto compute_nearest_idx = [](const ExprHandle& scale,
                                 const ExprHandle& dst_index,
                                 const ExprHandle& input_size) {
     return Min::make(
@@ -731,8 +705,8 @@ Tensor computeUpsampleNearest2d(
       "upsample_nearest2d",
       outputShape,
       Dtype(*outputType),
-      c10::nullopt, // initializer
-      fmap(strides, [&](ExprPtr stride) { return ExprHandle(stride); }),
+      std::nullopt, // initializer
+      fmap(strides, [&](const ExprPtr& stride) { return ExprHandle(stride); }),
       ExprHandle(A.node()->qscale()),
       ExprHandle(A.node()->qzero()));
   return Tensor(buf, args, e);
@@ -742,7 +716,7 @@ Tensor computeUpsampleNearest2dExternalCall(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
   Dtype dtype = kFloat;
   if (outputType) {
@@ -750,18 +724,18 @@ Tensor computeUpsampleNearest2dExternalCall(
   }
   int64_t output_size_h = -1;
   int64_t output_size_w = -1;
-  if (auto output_sizes = c10::get_if<IntList>(&inputs[1])) {
+  if (auto output_sizes = std::get_if<IntList>(&inputs[1])) {
     output_size_h = (*output_sizes)[0];
     output_size_w = (*output_sizes)[1];
   }
 
   double scale_factor_h = -1.f;
   double scale_factor_w = -1.f;
-  if (auto scale_factors = c10::get_if<DoubleList>(&inputs[2])) {
+  if (auto scale_factors = std::get_if<DoubleList>(&inputs[2])) {
     scale_factor_h = (*scale_factors)[0];
     scale_factor_w = (*scale_factors)[1];
   }
-  const BufHandle& x = c10::get<BufHandle>(inputs[0]);
+  const BufHandle& x = std::get<BufHandle>(inputs[0]);
   double qx_qscale = -1.f;
   int64_t qx_qzero = -1l;
   int64_t qx_qdtype = -1l;
@@ -801,10 +775,9 @@ Tensor computeQuantizedSigmoidExternalCall(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    // NOLINTNEXTLINE
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device) {
-  const BufHandle& qx = c10::get<BufHandle>(inputs[0]);
+  const BufHandle& qx = std::get<BufHandle>(inputs[0]);
 
   const auto out_qdtype = immQDType(qx);
   const double out_qscale = 1.0f / 256.0f;
@@ -834,6 +807,4 @@ Tensor computeQuantizedSigmoidExternalCall(
   return Tensor(ResultBuf.node(), s);
 }
 
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr
